﻿%% FILE ON: /zim8/dcserv/services/dcdocs/zimboot3.cmd
%%
%% Comments: Included new Entity Sets, Relationships and Roles to support 
%%           new report painter & report generator: 
%%           Templates, TemplFields, _RepGroups and RepItems.
%%           TemplastesTemplFields, TemplatesReports, _ReportsRepGroups, 
%%           ReportsRepItems, _RepGroupsRepItems and TemplFieldsRepItems.
%%           TplS  --> Templates
%%           TplFs --> TemplFields
%%           _RPTS --> _Reports
%%           _RPTG --> _RepGroups
%%           _RPTI --> _RepItems
%%           Updated the Entity Set _Reports to support this update above.
%%           Alexandre Macedo - July, 2009
%% Attention!
%%           Using new values to enumerate the field $SN (from 8.380.000).
%%           ZIM has limit to 8388608 for this field.
%%
%%           - 8380000 - Entity Sets
%%           - 8385000 - Roles
%%           - 8386000 - RelationShips
%%
%% Procedure to create new entitysets:
%% - First : add new entityset.
%% - Second: add all fields related with this new entityset.
%% - Third : adjust permissions, delete definitions of the entityset and his
%%           fields and finally create new entityset
%%
%% Have no obligation here to execute command [CREATE FIELD]!
%%
%% plAddConsts: Update Procedure to create new constants without:
%%  - Deleting definitons from ZIM Data Dictionary
%%  - Using $SN function
%%========================================================================%

%%========================================================================%
%% Break Handler Routine
%%========================================================================%
localprocedure plBreakHandler(inout vlBreakStatus )
   let vlBreakStatus = $true 
endprocedure
<< FR 12/05/2004 replaced by $exist$.
localprocedure plCheckForDataFile(vlObjectName) \
      local (vlSet)
   let $LastErrCode = 0 \ % Returns $lasterrcode
       vlSet = $SetProperty('Session', 'Error Trace', 'no') \
       <9> = $trim(vlObjectName)
   set messages off
   find 1 #<9>
   set messages on
   let vlSet = $SetProperty('Session', 'Error Trace', 'yes')
endprocedure
>>

%%========================================================================%
%% Adding EntitySets
%% --> Just EntitySets are added on this local procedure!
%%========================================================================%
localprocedure LPAddEnt( inout vlBreakStatus, \
   vlOwnerNm )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <OwnerNm> = $trim( vlOwnerNm )
       
   if $exist$( 'ZIM', '#<OwnerNm>', 4 ) = $TRUE
      if $sysdirid(ZIM.#<OwnerNm>) = 1 % i.e. in ZIM dir
         RETURN
      endif
   endif

   add 1 EntitySets \
      let \
         EntitySets.EntName = '#<OwnerNm>' \
         EntitySets.EntType = '' \
         EntitySets.AvgSize = '0' \
         EntitySets.DirName = 'ZIM'
  
endprocedure

%%========================================================================%
%% Adding Fields
%% Just FIELDS are add on this local procedure!
%%========================================================================%
localprocedure LPAddField( inout vlBreakStatus, \
   vlOwnerNm, vlFldName, vlDataType, vlLength, vlSN, vlSeq, \
   vlIndex, vlReqd, vlVirtual, vlDefault )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <SN> = $trim( vlSN ) \
       <Seq> = $trim( vlSeq ) \
       <OwnerNm> = $trim( vlOwnerNm ) \
       <FldName> = $trim( vlFldName ) \
       <DataType> = $trim( vlDataType ) \
       <Length> = $trim( vlLength ) \
       <Index> = { 'pkey' where $tolower(vlIndex) = 'pkey', \
                   'unique' where $tolower(vlIndex) = 'unique', \
                   'yes' where $tolower(vlIndex) = 'yes', \
                   'no' } \
       <Reqd> = { 'ro' where $tolower(vlReqd) = 'ro', \
                  'roe' where $tolower(vlReqd) = 'roe', \
                  'yes' where $tolower(vlReqd) = 'yes', \
                  'no' } \
       <Virtual> = { 'yes' where $tolower(vlVirtual) = 'yes', 'no' } \
       <Default> = $trim( vlDefault )
   let <ListWidth> = {'-1' where $tolower('#<Virtual>') = 'yes' or $tolower('#<FldName>') in ('objectkey', 'objecttype', 'nullobjectkey'), '#<Length>' }

   if( $exist$( '#<OwnerNm>', '#<FldName>', 5 ) = $TRUE )
      if( $sysdirid( #<OwnerNm>.#<FldName> ) = 1 ) %% i.e. in ZIM dir
         RETURN
      endif
   endif

   add 1 Fields \
      let \
         Fields.SN           = '#<SN>' \
         Fields.FieldName    = '#<FldName>' \
         Fields.OwnerName    = '#<OwnerNm>' \
         Fields.Type         = '#<DataType>' \
         Fields.Length       = '#<Length>' \
         Fields.Decimals     = '0' \
         Fields.Reqd         = '#<Reqd>' \
         Fields.Index        = '#<Index>' \
         Fields.Virtual      = '#<Virtual>' \
         Fields.ListWidth    = '#<ListWidth>' \
         Fields.FieldHeading = $null \
         Fields.FieldMask    = $null \
         Fields.Default      = "#<Default>" where "#<Default>" <> '' \
         Fields.EQSel        = '1.000' \
         Fields.BWSel        = '1.000' \
         Fields.LGSel        = '1.000' \
         Fields.DirName      = 'ZIM'
   
endprocedure

%%========================================================================%
%% Adding Fields
%%========================================================================%
localprocedure plFieldAdd( inout vlBreakStatus, \
   vlOwnerNm, vlFldName, vlDataType, vlLength, vlSN, vlSeq, \
   vlIndex, vlReqd, vlVirtual, vlDefault )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <SN> = $trim( vlSN ) \
       <Seq> = $trim( vlSeq ) \
       <OwnerNm> = $trim( vlOwnerNm ) \
       <FldName> = $trim( vlFldName ) \
       <DataType> = $trim( vlDataType ) \
       <Length> = $trim( vlLength ) \
       <Index> = { 'unique' where $tolower(vlIndex) = 'unique', \
                   'yes' where $tolower(vlIndex) = 'yes', \
                   'no' } \
       <Reqd> = { 'yes' where $tolower(vlReqd) = 'yes', 'no' } \
       <Virtual> = { 'yes' where $tolower(vlVirtual) = 'yes', 'no' } \
       <Default> = $trim( vlDefault )
   let <ListWidth> = {'-1' where $tolower('#<Virtual>') = 'yes' or $tolower('#<FldName>') in ('objectkey', 'objecttype', 'nullobjectkey'), '#<Length>' }

   if( $exist$( '#<OwnerNm>', '#<FldName>', 5 ) = $TRUE )
      if( $sysdirid( #<OwnerNm>.#<FldName> ) = 1 ) %% i.e. in ZIM dir
         out '#<OwnerNm>' '#<FldName>' "already exists..."
         RETURN
      endif
   endif

   add 1 Fields \
      let \
         Fields.SN           = '#<SN>' \
         Fields.FieldName    = '#<FldName>' \
         Fields.OwnerName    = '#<OwnerNm>' \
         Fields.Type         = '#<DataType>' \
         Fields.Length       = '#<Length>' \
         Fields.Decimals     = '0' \
         Fields.Reqd         = '#<Reqd>' \
         Fields.Index        = '#<Index>' \
         Fields.Virtual      = '#<Virtual>' \
         Fields.ListWidth    = '#<ListWidth>' \
         Fields.FieldHeading = $null \
         Fields.FieldMask    = $null \
         Fields.Default      = "#<Default>" where "#<Default>" <> '' \
         Fields.EQSel        = '1.000' \
         Fields.BWSel        = '1.000' \
         Fields.LGSel        = '1.000' \
         Fields.DirName      = 'ZIM'
         
   create Field #<OwnerNm> #<FldName> in ZIM $sn #<Seq>

   %% Set permissions
   if '#<Virtual>' <> $toalpha('yes',3)
      permission #<Ownernm>.#<FldName> owner read update
      permission #<Ownernm>.#<FldName> group read update
      permission #<Ownernm>.#<FldName> other read update
   else %% virtual fields are read only of course
      permission #<Ownernm>.#<FldName> owner read
      permission #<Ownernm>.#<FldName> group read
      permission #<Ownernm>.#<FldName> other read
   endif

   delete 1 Fields \
      where \
         Fields.FieldName = '#<FldName>' and \
         Fields.OwnerName = '#<OwnerNm>' and \
         Fields.DirName = 'ZIM'

endprocedure

%%========================================================================%
%% EntitySet: create, set permissions and delete its definition
%% Fields: set Permissions and delete its definition
%% ==> ZIM Data Dictionary has no definition fields available
%%========================================================================%
localprocedure LPCreateEnt( inout vlBreakStatus, \
   vlOwnerNm, vlSeq, vlFileNum ) \
   local (vlcount)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <Seq>     = $trim( vlSeq ) \
       <FileNum> = $trim( vlFileNum ) \
       <OwnerNm> = $trim( vlOwnerNm )
       
   if $exist$( 'ZIM', '#<OwnerNm>', 4 ) = $TRUE
      if $sysdirid(ZIM.#<OwnerNm>) = 1 % i.e. in ZIM dir
         RETURN
      endif
   endif

@timezero 
   create EntitySet #<OwnerNm> in ZIM $fn #<FileNum> $sn #<Seq> delete

   find all fields \
      where fields.ownername = '#<Ownernm>' \
        and fields.dirname   = 'ZIM' -> sfields

   let vlcount = $membercount

   while (let vlcount=vlcount-1) >= 0
   
      let <FldName> = $trim( FieldName ) \
          <Virtual> = { 'yes' where $tolower(Virtual) = 'yes', 'no' }
   
      %% Set permissions
      if '#<Virtual>' <> $toalpha('yes',3)
         permission ZIM.#<Ownernm>.#<FldName> owner read update
         permission ZIM.#<Ownernm>.#<FldName> group read update
         permission ZIM.#<Ownernm>.#<FldName> other read update
      else %% virtual fields are read only of course
         permission ZIM.#<Ownernm>.#<FldName> owner read
         permission ZIM.#<Ownernm>.#<FldName> group read
         permission ZIM.#<Ownernm>.#<FldName> other read
      endif
   
      delete 1 Fields \
         where Fields.FieldName = '#<FldName>' \
           and Fields.OwnerName = '#<OwnerNm>' \
           and Fields.DirName   = 'ZIM'
           
      set currentset sfields
      next sfields
       
   endwhile      

   %% Set permissions
   permission ZIM.#<OwnerNm> owner read add change delete
   permission ZIM.#<OwnerNm> group read add change delete
   permission ZIM.#<OwnerNm> other read add change delete

   delete 1 Ents \
      where Ents.EntName = '#<OwnerNm>' and Ents.DirName = 'ZIM'

endprocedure

%%========================================================================%
%% Adding Constants
%%========================================================================%
localprocedure plAddConsts( inout vlBreakStatus, \
                                 vlConstName, \
                                 vlDataType, \
                                 vlValue)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <ConstName> = $trim( vlConstName ) \
       <DataType> = $trim( vlDataType ) \
       <ConstValue> = $trim( vlValue ) 

   if $exist$( 'ZIM', '#<ConstName>', 18 ) = $FALSE
      add 1 Consts \
         let \
            Consts.ConstName = '#<ConstName>' \
            Consts.DirName = 'ZIM' \
            Consts.ConstType = '#<DataType>' \
            Consts.ConstValue = '#<ConstValue>'
      create constant #<ConstName> in ZIM
   endif
endprocedure

%%========================================================================%
%% Updating Constants
%%========================================================================%
localprocedure plConstUpdate( inout vlBreakStatus, \
                                    vlConstName, \
                                    vlDataType, \
                                    vlValue, \
                                    vlSeq)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <ConstName> = $trim( vlConstName ) \
       <DataType> = $trim( vlDataType ) \
       <ConstValue> = $trim( vlValue ) \
       <Seq> = $trim( vlSeq )
   if $exist$( 'ZIM', '#<ConstName>', 18 ) = $true
      delete 1 Consts \
         where Consts.ConstName = '#<ConstName>' and Consts.DirName = 'ZIM'
@syser
      erase constant ZIM.#<ConstName> from ZIM
   endif

   add 1 Consts \
      let \
         Consts.ConstName = '#<ConstName>' \
         Consts.DirName = 'ZIM' \
         Consts.ConstType = '#<DataType>' \
         Consts.ConstValue = '#<ConstValue>'
   create constant #<ConstName> in ZIM $sn #<Seq>

   delete 1 Consts \
      where Consts.ConstName = '#<ConstName>' and Consts.DirName = 'ZIM'
endprocedure

%%========================================================================%
%% Adding Documents
%%========================================================================%
localprocedure plDocAdd( inout vlBreakStatus, vlDocName, \
      vlFileName, vlDocType, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <DocName> = $trim( vlDocName ) \
       <FileName> = $trim( vlFileName ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<DocName>', 9 ) = $FALSE
      add 1 Docs \
         let \
            Docs.DocName = '#<DocName>' \
            Docs.FileName = '#<FileName>' \
            Docs.DocType = $trim(vlDocType) \
            Docs.DirName = 'ZIM'
      create document #<DocName> in ZIM $sn #<Seq>
   endif
endprocedure

%%========================================================================%
%% Adding Roles
%%========================================================================%
localprocedure plRoleAdd( inout vlBreakStatus, \
                                vlRoleName, \
                                vlRoleOwner, \
                                vlRoleCondition, \
                                vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <RoleName> = $trim( vlRoleName ) \
       <RoleOwner> = $trim( vlRoleOwner ) \
       <RoleCondition> = $trim( vlRoleCondition ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<RoleName>', 12 ) = $FALSE
      add 1 Roles \
         let \
            Roles.RoleName = '#<RoleName>' \
            Roles.OwnerName = '#<RoleOwner>' \
            Roles.RoleCondition = '#<RoleCondition>' \
            Roles.DirName = 'ZIM'
      create role #<RoleName> in ZIM $sn #<Seq>
      delete Roles \
         where Roles.RoleName = '#<RoleName>' and Roles.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Adding Relationships
%%========================================================================%
localprocedure plRelAdd( inout vlBreakStatus, vlRelName, vlRelExpr, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <RelName> = $trim( vlRelName ) \
       <RelExpr> = $trim( vlRelExpr ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<RelName>', 6 ) = $FALSE        %% NODATAREL !!
      add 1 Rels \
         let \
            Rels.RelName = '#<RelName>' \
            Rels.RelCondition = '#<RelExpr>' \
            Rels.DirName = 'ZIM'
      create relationship #<RelName> in ZIM $sn #<Seq> delete

      delete 1 Rels \
         where Rels.RelName = '#<RelName>' and Rels.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Adding Directories
%%========================================================================%
localprocedure plDirAdd( inout vlBreakStatus, vlDirName, vlDirID, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <DirName> = $trim( vlDirName ) \
       <DirID> = $trim( vlDirID ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<DirName>', 2 ) = $FALSE
      add 1 Directories \
         let \
            Directories.DirName = '#<DirName>' \
            Directories.OwnerDirName = 'ZIM' \
            Directories.DirId = '#<DirID>' \
            Directories.Foreign = 'yes'
      create Directory #<DirName> in ZIM $sn #<Seq> delete

      %% Set permissions
      permission ZIM.#<DirName> owner read update
      permission ZIM.#<DirName> group read update
      permission ZIM.#<DirName> other read update

      delete 1 Dirs \
         where Dirs.DirName = '#<DirName>' and Dirs.OwnerDirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%       Program to add special Event, Window, Form, and Context
%               structures. These are created as incore entity sets.
%               They must be created in the ZIM dir with system names
%               less than 1000 and will be sorted by SN which should
%               be set to the desired system name.
%
%       Currently assigned system name ranges are:
%
%       Event           - 900, fields 901 to 915
%       ThisWindow      - 920, fields 921 to 938
%       ThisForm        - 940, fields 941 to 958
%       ThisMenu        - 970, fields 971 to 975
%%========================================================================%
localprocedure  plAddStructureFld( pEntName, pFieldName, pDataType, \
                                        pLength, pSN )

add fields let \
        SN = pSN \
        FieldName = pFieldName \
        OwnerName = pEntName \
        Type = pDataType \
        Length = pLength \
        Decimals = 0 \
        DirName = 'ZIM'

endprocedure

%%======================================================================================%
% Updates the communication between ZimIDE, ODBC Driver and ZOM.
%%======================================================================================%
localprocedure UpdateODBCAndZOM()

    % Remove previous attempts to do the trick.
    delete RemoteProcedures where RemoteProcName="pZOMCall" and DirName="ZIM"
    delete all ResultFields where RemoteProcName="pZOMCall" and DirName="ZIM"
    delete all ResultSets where RemoteProcName="pZOMCall" and DirName="ZIM"

    % Add all needed fields.
    add 1 RemoteProcedures let RemoteProcName="pZOMCall" NumInputParams=1 NumOutputParams=2 NumResultSets=0 DocName="pZOMCall" DirName="ZIM"
    add 1 ResultFields let RemoteProcName="pZOMCall" ResultName="Action" ResultType=1 Length=256 Decimals=0 Reqd="Yes" Type="Alpha" OwnerName="" DirName="ZIM" SN=10 SetSN=-1
    add 1 ResultFields let RemoteProcName="pZOMCall" ResultName="ErrCode" ResultType=4 Length=4 Decimals=0 Reqd="Yes" Type="Alpha" OwnerName="" DirName="ZIM" SN=20 SetSN=-1
    add 1 ResultFields let RemoteProcName="pZOMCall" ResultName="ErrMsg" ResultType=4 Length=256 Decimals=0 Reqd="Yes" Type="Alpha" OwnerName="" DirName="ZIM" SN=30 SetSN=-1
    add 1 ResultFields let RemoteProcName="pZOMCall" ResultName="Lines" ResultType=3 Length=256 Decimals=0 Reqd="Yes" Type="Alpha" OwnerName="Logues" DirName="ZIM" SN=40 SetSN=10
    add 1 ResultSets let RemoteProcName="pZOMCall" SetName="Logues" DirName="ZIM" SetSN=10
endprocedure

%%========================================================================%
% Upgrading from 5.3 to 1.0 (if necessary)
%%========================================================================%
localprocedure plDCv53r10Update( inout vlBreakStatus ) \
   local(vlExists)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   compute 1 EventAction where RecId = 12 and EventActionOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding a System EventAction ...'
      add 1 EventAction \
         let WindowTag = '_$wSpy' ObjectTag = 'Close' \
             EventName = 'Click' EventType = 'FormField' \
             RecId = 12 EventActionOwner = 'System' \
             Action = 'Spy_Close' Macro = "$DeployServices.pSpy('CLOSE')"
   endif

   compute 1 EventAction where RecId = 282 and EventActionOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding a System EventAction ...'
      add 1 EventAction \
         let WindowTag = 'Unknown' ObjectTag = 'Unknown' \
             EventName = 'Break'  EventType = 'System' \
             RecId = 282 EventActionOwner = 'System' \
             Comment = 'Break events while in pGetAction.' \
             Action = 'Abort'
   endif

%%   output 'Updating ApplMessages ...'
%% Check for 5.2 ApplMessages
%%   plCheckForDataFile('$DeployServices.ApplMessages_52') FR 12/05/2004
%%   if $LastErrCode = 0 % Convert the old ones
%%     if $Exist$('$DeployServices','ApplMessages_52',4) = 0
%%     delete all ApplMessages
%%      add all ApplMessages from ApplMessages_52 \
%%         let ApplMessCode = $leftjustify(ApplMessages_52.ApplMessCode)
%%      set escapechar off
%%      change 1 ApplMessages where ApplMessCode = '31' and ApplMessType = 'I' \
%%         let ApplMessType = 'W' \
%%             ApplMessStyle = 4 \
%%             ApplMessage = '&1 are required for &2.\rAdd &1 now?'
%%      set escapechar on
%%      if $membercount = 1 % The updates haven't been done yet
%%         change 1 ApplMessages \
%%            where ApplMessCode = pmErrNotFound and ApplMessType = 'I' \
%%            let ApplMessType = 'E' \
%%                ApplMessStyle = 7
%%         add 1 ApplMessages from ApplMessages where ApplMessCode = '25' \
%%            let ApplMessCode = '32' \
%%                ApplMessage = 'Changing &1 ...'
%%         add 1 ApplMessages from ApplMessages where ApplMessCode = '26' \
%%            let ApplMessCode = '33' \
%%                ApplMessage = 'Deleting &1 ...'
%%         add 1 ApplMessages from ApplMessages where ApplMessCode = '27' \
%%            let ApplMessCode = '34' \
%%                ApplMessage = 'Adding &1 ...'
%%      endif
%%   endif

endprocedure

%%========================================================================%
% Upgrading ZomInit
%%========================================================================%
localprocedure plUpdateZOMInit(inout vlBreakStatus) \
   local (vlMacro)

on error
   goto next
endon

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   set save
   set messages off

   find 1 $ZOM.pZOMInit
   if $setcount = 1
      if not vZOMEnabled = $true % Try to get any current settings
         compute 1 $zom.pZOMInit where Text like '%vZOMInstanceID%=%' \
            evaluate (let vlMacro = $concat({'' where Text like '%let%', 'let '}, \
                                            $translate(Text, '\\', '')))
         if $membercount = 1
%%            output 'Updating ZOM initialization script ...'
            let <MacroToExecute> = vlMacro
            #<MacroToExecute>
            let <Property> = $concat('INI:[ZOM for ', $trim(vZOMInstanceID), ']')
            let vZOMDefaultDir = \
                   $getproperty('#<Property>', 'Default ZIM Directory')
            if vZOMDefaultDir is not $null % There are INI settings
               let vZOMChgKeyword = \
                      $getproperty('#<Property>', 'Changed Keyword') \
                   vZOMConventionsOn = \
                      $getproperty('#<Property>', 'Conventions On') \
                   vZOMDepTrace = $getproperty('#<Property>', 'Dependency Trace') \
                   vZOMDisplay = $getproperty('#<Property>', 'Display Mode') \
                   vZOMErrorLog = $getproperty('#<Property>', 'Log Errors') \
                   vZOMExplodeLevel = \
                      $getproperty('#<Property>', 'Explode Level') \
                   vZOMTurbo = $getproperty('#<Property>', 'Express Mode') \
                   vZOMNewKeyword = $getproperty('#<Property>', 'New Keyword') \
                   vZOMSortOrder = $getproperty('#<Property>', 'Sort Order')
            else % Try to extract them from the pre-Version 5.4 pZOMInit file
               compute 1 $zom.pZOMInit where Text like '%vZOMDisplay%=%' \
                  evaluate (let vZOMDisplay = \
                     {'N' where Text like '%=%0%' or Text like '%=%N%', \
                      'S' where Text like '%=%2%' or Text like '%=%S%', 'W'})
               compute 1 $zom.pZOMInit where Text like '%vZOMErrorLog%=%' \
                  evaluate (let vZOMErrorLog = \
                     {$true where Text like '%=%$true%', $false})
               compute 1 $zom.pZOMInit where Text like '%vZOMConventionsOn%=%' \
                  evaluate (let vZOMConventionsOn = \
                     {$true where Text like '%=%$true%', $false})
               compute 1 $zom.pZOMInit where Text like '%vZOMExplodeLevel%=%' \
                  evaluate (let vZOMExplodeLevel = \
                     $substring(Text, $position(Text, '=') + 1, 2))
               compute 1 $zom.pZOMInit where Text like '%vZOMDepTrace%=%' \
                  evaluate (let vZOMDepTrace = \
                     {$true where Text like '%=%$true%', $false})
               compute 1 $zom.pZOMInit where Text like '%vZOMTurbo%=%' \
                  evaluate (let vZOMTurbo = \
                     {$true where Text like '%=%$true%', $false})
               compute 1 $zom.pZOMInit where Text like '%vZOMDefaultDir%=%' \
                  evaluate (let vlMacro = \
                     $concat({'' where Text like '%let%', 'let '}, \
                             $translate(Text, '\\', '')))
               if $membercount = 1
                  let <MacroToExecute> = vlMacro
                  #<MacroToExecute> vlMacro = ''
               endif
               compute 1 $zom.pZOMInit where Text like '%vZOMNewKeyword%=%' \
                  evaluate (let vlMacro = $concat({'' where Text like '%let%', 'let '}, \
                                                  $translate(Text, '\\', '')))
               if $membercount = 1
                  let <MacroToExecute> = vlMacro
                  #<MacroToExecute> vlMacro = ''
               endif
               compute 1 $zom.pZOMInit where Text like '%vZOMChgKeyword%=%' \
                  evaluate (let vlMacro = $concat({'' where Text like '%let%', 'let '}, \
                                                  $translate(Text, '\\', '')))
               if $membercount = 1
                  let <MacroToExecute> = vlMacro
                  #<MacroToExecute> vlMacro = ''
               endif
            endif
         endif
      endif
   endif
   pZOMConfig('RESET')
   set restore

endprocedure

%%========================================================================%
% Upgrading from 5.3 to 3.1 (if necessary)
%%========================================================================%
localprocedure plDCv53r31Update(inout vlBreakStatus)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%   output 'Updating ApplMessages ...'
   set escapechar off
   change 1 ApplMessages where ApplMessKey = 'E  -9000' \
      let ApplMessage = \
        'A process (Id: "&1", Window: "&2")\rwas not found in the process list.'
   change 1 ApplMessages where ApplMessKey = 'E  2010' \
      let ApplMessage = $translate(ApplMessage, '?.', '?')
   set escapechar on

%%   output 'Updating ApplMessTitles ...'
%% Check for 5.2 ApplMessTitles
%%   plCheckForDataFile('$DeployServices.ApplMessTitles_52')
%%   if $LastErrCode = 0 % Convert the old ones
%%   if $Exist$('$DeployServices','ApplMessTitles_52',4) = 0
%%      delete all ApplMessTitles
%%      add all ApplMessTitles from ApplMessTitles_52
%%   endif

   plUpdateZOMInit(vlBreakStatus)

endprocedure

%%========================================================================%
% Upgrading from 5.4 to 1.1 (if necessary)
%%========================================================================%
localprocedure plDCv54r11Update( inout vlBreakStatus ) \
   local (vlExists)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%   output 'Updating a System EventAction ...'
   change 1 EventAction where RecId = 80 and EventActionOwner = 'System' \
         let Action = 'ToolBarHide' Macro = ''

   pDPSFileDelete(')sysset.cfg')

endprocedure

%%========================================================================%
% Upgrading from 5.4 to 2.1 (if necessary)
%%========================================================================%
localprocedure plDCv54r21Update( inout vlBreakStatus ) \
   local (vlExists, vlDisabled)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%   output 'Turning off "Definitions Logging" ...'
   set logdefinitions off
   pSystemSettings('Disable', '', '')
   pSystemSettings('Load', '', '')
   pSystemSettings('Set', 'LogDefinitions', 'off')
   pSystemSettings('Save', '', '')

   compute 1 EventAction where RecId = 640 and EventActionOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding a new System EventAction ...'
      add 1 EventAction \
         let WindowTag = 'Unknown'  ObjectTag = 'TabPage' \
             EventName = 'GotFocus' EventType = 'FormField' \
             RecId = 640 EventActionOwner = 'System' \
             Comment = 'A Tab in a Tabbed Dialog was selected.' \
             Action = '<FieldTag>'
   endif

   compute 1 EventAction where RecId = 650 and EventActionOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding an additional 3 System EventAction...'

      add 1 EventAction \
         let WindowTag = 'wSEC_Permissions'  ObjectTag = 'ObjectType' \
             EventName = 'Modified' EventType = 'FormField' \
             RecId = 650 EventActionOwner = 'System' \
             Comment = 'Overrides the SetModied Action for modified events.' \
             Action = '<FieldTag>'

      add 1 EventAction \
         let WindowTag = 'wSEC_Permissions'  ObjectTag = 'OwnerChange' \
             EventName = 'Modified' EventType = 'FormField' \
             RecId = 660 EventActionOwner = 'System' \
             Comment = 'Overrides the SetModied Action for modified events.' \
             Action = '<FieldTag>'

      add 1 EventAction \
         let WindowTag = 'wSEC_Permissions'  ObjectTag = 'ObjectNameList' \
             EventName = 'Modified' EventType = 'FormField' \
             RecId = 670 EventActionOwner = 'System' \
             Comment = 'Overrides the SetModied Action for modified events.' \
             Action = '<FieldTag>'

   endif

   compute 1 ObjectAction where RecId = 1630 and ObjectActionOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding 3 new System ObjectActions ...'

   add 1 ObjectAction \
      let WindowTag = 'ObjList$SWW' \
         Action = 'New' \
         State = 'Unknown' \
         View = 'Unknown' \
         ValidAction = 'No' \
         NewState = 'Unknown'\
         NewView = 'Unknown' \
         ObjectActionOwner = 'System' \
         SaveData = '' \
         RecId = 1630

   add 1 ObjectAction \
      let WindowTag = 'ObjList$SWW' \
         Action = 'Delete' \
         State = 'Unknown' \
         View = 'Unknown' \
         ValidAction = 'No' \
         NewState = 'Unknown' \
         NewView = 'Unknown' \
         ObjectActionOwner = 'System' \
         SaveData = '' \
         RecId = 1640

   add 1 ObjectAction \
      let WindowTag = 'ObjList$SWW' \
         Action = 'Duplicate' \
         State = 'Unknown' \
         View = 'Unknown' \
         ValidAction = 'No' \
         NewState = 'Unknown' \
         NewView = 'Unknown' \
         ObjectActionOwner = 'System' \
         SaveData = '' \
         RecId = 1650

   endif
   
%%   output 'Changing a System EventAction...'
   change EventAction where RecID = 10000 let RecID = 680

%%   output 'Creating the "Port" directory ...'
   if $zimos = osWin3
      system $concat("!mkdir ", $DBPath, "\\port ") invisible remote
   else %% unix
      system $concat("mkdir -p ", $DBPath, "/port ") remote
   endif   
   
endprocedure

%%========================================================================%
% Upgrading from 5.5 to 1.1 (if necessary)
%%========================================================================%
localprocedure plDCv55r11Update( inout vlBreakStatus ) \
   local (vlExists, vlDisabled)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   plUpdateZOMInit(vlBreakStatus)

   compute 1 EventAction where RecId = 681 and EventActionOwner = 'System' \
      evaluate ('')

   compute 1 ApplMessTitles \
      where ApplOperation = 'EXPORT' and ApplMessTitleOwner = 'System' \
      evaluate ('')
   if $membercount = 0
%%      output 'Adding 2 System Application Message Titles...'

      add 1 ApplMessTitles \
        let ApplOperation = 'EXPORT'\
            ApplMessTitle = 'Export' \
            ApplMessTitleOwner = 'System' \
            ApplMessTitleLang = 'E'

      add 1 ApplMessTitles \
        let ApplOperation = 'IMPORT'\
            ApplMessTitle = 'Import' \
            ApplMessTitleOwner = 'System' \
            ApplMessTitleLang = 'E'
   endif

endprocedure


%%========================================================================%
% Upgrading from 6.1 to 1.0 (if necessary)
%%========================================================================%
localprocedure plDCv61r10Update( inout vlBreakStatus )


on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%TODO: Add update code here

%%   output 'Converting ZOF Classes to Zim Classes...'

   find all Docs where DocType = 'ZOF Class' -> sZOFClassSet
   if $setcount > 0
      change all let DocType = 'Zim Class'
      top sZOFClassSet
      while
         Let <DirName> = $trim(Docs.DirName) \
             <ZOFClassDoc> = $trim(Docs.DocName)
         if $compilestatus(#<DirName>.#<ZOFClassDoc>) = 1
            zomrecreate #<DirName>.#<ZOFClassDoc>
            % Do nothing since document is not compiled.
         elseif $compilestatus(#<DirName>.#<ZOFClassDoc>) = 2
            zomrecreate #<DirName>.#<ZOFClassDoc>
%%            output '#<DirName>.#<ZOFClassDoc> not compiled due to missing source file.'
         elseif $compilestatus(#<DirName>.#<ZOFClassDoc>) = 7
            zomrecreate #<DirName>.#<ZOFClassDoc>
%%            output '#<DirName>.#<ZOFClassDoc> not compiled due to missing dependant object.'
         else
            zomrecreate #<DirName>.#<ZOFClassDoc>
            zomcompile #<DirName>.#<ZOFClassDoc>
         endif
         if $currentmember('sZOFClassSet') = $lastmember('sZOFClassSet')
            break
         else
            set currentset sZOFClassSet
            next sZOFClassSet
         endif
      endwhile
   endif

endprocedure


%% TODO 1: Copy this template and insert above
%%         Change NEWVERSION and NEWREVISION

<<

localprocedure plDCvNEWVERSIONrNEWREVISIONUpdate( inout vlBreakStatus )


on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%TODO: Add update code here

endprocedure

>>

%%========================================================================%
% Updating ZIM 8.2 Rev 3.1 - Alexandre Macedo - July/2009 
%
%  Program to add special tables to support the new REPORT DESIGNER:
%   _Reports, _REPGROUPS, _REPITEMS, TEMPLATES and TEMPLFIELDS.
%
%   These are created as incore entity sets.
%
%   They must be created in the ZIM dir with system names greater than
%   8.380.000 and will be sorted by SN which should be set to the desired 
%   system name.
%
%   Templates:          8380000-8380008
%   TemplFields:        8380050-8380069
%   _Reports:           8380100-8380139
%   _RepGroups:         8380150-8380161
%   _RepItems:          8380200-8380227
%   
%   TplS (Templates):   8385000
%   TplFs (TemplFields):8385001
%   _RPTS (_Reports):   8385002
%   _RPTG (_Reports):   8385003
%   _RPTI (_Reports):   8385004
%   
%   TplSTplFs           Templates.TplName=TemplFields.TplName   8386000
%   TplSReports         Templates.TplName=_Reports.TplName      8386001
%   _ReportsRepGroups   _Reports._RepName=_RepGroups._RepName   8386002
%   _ReportsRepItems    _Reports._RepName=_RepItems._RepName    8386003
%   _RepGroupsRepItems  _RepGroups._RepGKey=_RepItems._RepGKey  8386004
%   TplFsRepItems       TemplFields.TplFsKey=_RepItems.TplFsKey 8386005
%
%%========================================================================%
localprocedure plDCv82r31Update( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   output 'Adding Templates'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   LPAddEnt( vlBreakStatus, 'Templates' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'Templates', 'TplName', 'Alpha', '18', '1', '8380001', 'unique', 'yes', 'no', '' )

   LPAddField( vlBreakStatus, \
       'Templates', 'PgOrientation', 'Char', '1', '2', '8380002', 'no', 'no', 'no', "'P'" )

   LPAddField( vlBreakStatus, \
       'Templates', 'ModTime', 'Date', '8', '3', '8380003', 'no', 'no', 'no', '' )

   LPAddField( vlBreakStatus, \
       'Templates', 'Dirname', 'Alpha', '18', '4', '8380004', 'no', 'no', 'no', "'ZIM'" )

   LPAddField( vlBreakStatus, \
       'Templates', 'TplPageSize', 'VarChar', '60', '5', '8380005', 'no', 'no', 'no', "'rep$Letter'" )

   LPCreateEnt( vlBreakStatus, 'Templates', '8380000','0020' )

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   output 'Adding TemplFields'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   LPAddEnt( vlBreakStatus, 'TemplFields' )
   if( vlBreakStatus = $true )
      set exception break
   endif
   
   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplName', 'Alpha', '18', '1', '8380051', 'yes', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsName', 'Alpha', '18', '2', '8380052', 'yes', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsId', 'Int', '0', '3', '8380053', 'yes', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsPId', 'Int', '0', '4', '8380054', 'yes', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsGroup', 'Char', '3', '5', '8380055', 'no', 'no', 'no', "'DL'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsRow', 'Int', '0', '6', '8380056', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsCol', 'Int', '0', '7', '8380057', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsWdth', 'Int', '0', '8', '8380058', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsHght', 'Int', '0', '9', '8380059', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsFT', 'Char', '1', '10', '8380060', 'no', 'no', 'no', "'V'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsBC', 'Longint', '0', '11', '8380061', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsFC', 'Longint', '0', '12', '8380062', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsAlign', 'char', '2', '13', '8380063', 'no', 'no', 'no', "'TL'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplTxtOrient', 'Char', '1', '14', '8380064', 'no', 'no', 'no', "'H'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsZOrder', 'Int', '0', '15', '8380065', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsFontSize', 'Int', '0', '16', '8380066', 'no', 'no', 'no', '10' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsStyle', 'Int', '0', '17', '8380067', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsDM', 'Int', '0', '18', '8380068', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'Dirname', 'Alpha', '18', '19', '8380069', 'no', 'no', 'no', "'ZIM'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsFont', 'VarAlpha', '64', '20', '8380070', 'no', 'no', 'no', "'Courier'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsValue', 'VarAlpha', '256', '21', '8380071', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       'TemplFields', 'TplFsKey', 'Alpha', '36', '22', '8380072', 'yes', 'yes', 'yes', \
       '$concat(TemplFields.TplName,TemplFields.TplFsName)' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPCreateEnt( vlBreakStatus, 'TemplFields', '8380050','0021' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   output 'Adding _Reports'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   LPAddEnt( vlBreakStatus, '_Reports' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepName', 'Alpha', '18', '1', '8380101', 'yes', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepCompId', 'Int', '0', '2', '8380102', 'yes', 'yes', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPgSize', 'Int', '0', '3', '8380103', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPgWdth', 'Int', '0', '4', '8380104', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPgLines', 'Int', '0', '5', '8380105', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepTopMarg', 'Int', '0', '6', '8380106', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepBotMarg', 'Int', '0', '7', '8380107', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepLeftMarg', 'Int', '0', '8', '8380108', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepColSpace', 'Int', '0', '9', '8380109', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPause', 'Char', '1', '10', '8380110', 'no', 'no', 'no',"'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepDateFormat', 'Alpha', '8', '11', '8380111', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepGNewPage', 'Alpha', '1', '12', '8380112', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSGNewPage', 'Alpha', '1', '13', '8380113', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSNewPage', 'Alpha', '1', '14', '8380114', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepKeep', 'Alpha', '1', '15', '8380115', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepTitle', 'VarAlpha', '90', '16', '8380116', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepGExpr', 'VarAlpha', '256', '17', '8380117', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepGHeading', 'VarAlpha', '132', '18', '8380118', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSGExpr', 'VarAlpha', '256', '19', '8380119', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSGHeading', 'VarAlpha', '132', '20', '8380120', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSExpr', 'VarAlpha', '256', '21', '8380121', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSHeading', 'VarAlpha', '132', '22', '8380122', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepProgName', 'Alpha', '18', '23', '8380123', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPFileName', 'VarAlpha', '256', '24', '8380124', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepPDirName', 'Alpha', '18', '25', '8380125', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepODocName', 'Alpha', '18', '26', '8380126', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepOFileName', 'VarAlpha', '256', '27', '8380127', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepODirName', 'Alpha', '18', '28', '8380128', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepDate', 'Date', '8', '29', '8380129', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepNumToDo', 'Char', '10', '30', '8380130', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepSummarized', 'Char', '1', '31', '8380131', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepOutput', 'Alpha', '1', '32', '8380132', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_Reports', '_RepId', 'Int', '0', '33', '8380133', 'unique', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', 'TplName', 'Alpha', '18', '34', '8380134', 'yes', 'no', '0', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', '_RepAuthor', 'Alpha', '18', '35', '8380135', 'no', 'no', '0', "'ZIM'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', '_RepFROM', 'VarAlpha', '256', '36', '8380136', 'no', 'no', '0', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', '_RepWHERE', 'VarAlpha', '256', '37', '8380137', 'no', 'no', '0', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', '_RepSORT', 'VarAlpha', '256', '38', '8380138', 'no', 'no', '0', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \ %% New field Added July/2009
       '_Reports', 'ModTime', 'Date', '8', '39', '8380139', 'no', 'no', '0', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPCreateEnt( vlBreakStatus, '_Reports', '8380100','0022' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   output 'Adding _RepGroups'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   LPaddEnt( vlBreakStatus, '_RepGroups' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepName', 'Alpha', '18', '1', '8380151', 'yes', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepId', 'Int', '0', '2', '8380152', 'yes', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGType', 'Char', '3', '3', '8380153', 'no', 'no', 'no', "'DL'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGLevel', 'Int', '0', '4', '8380154', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGNewPage', 'Alpha', '1', '5', '8380155', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGExpr', 'VarAlpha', '256', '6', '8380156', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGHeading', 'VarAlpha', '132', '7', '8380157', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGWhere', 'VarAlpha', '256', '8', '8380158', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGOrientation', 'Char', '1', '9', '8380159', 'no', 'no', 'no', "'F'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', 'Dirname', 'Alpha', '18', '10', '8380160', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepGroups', '_RepGKey', 'Char', '23', '11', '8380161', 'Unique', 'yes', 'yes', \
       "$concat(_RepGroups._RepName,$mask(_RepGroups._RepId,'99999'))" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPCreateEnt( vlBreakStatus, '_RepGroups', '8380150','0023' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   output 'Adding _RepItems'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   LPAddEnt( vlBreakStatus, '_RepItems' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepName', 'Alpha', '18', '1', '8380201', 'yes', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepId', 'Int', '0', '2', '8380202', 'yes', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIId', 'Int', '0', '3', '8380203', 'yes', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', 'TplName', 'Alpha', '18', '4', '8380204', 'yes', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', 'TplFsName', 'Alpha', '18', '5', '8380205', 'yes', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIColNum', 'Int', '0', '6', '8380206', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIWdth', 'Int', '0', '7', '8380207', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepINewPage', 'Char', '1', '8', '8380208', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepINewLine', 'Int', '0', '9', '8380209', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepILine', 'Int', '0', '10', '8380210', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIOldLine', 'Char', '1', '11', '8380211', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepICentre', 'Char', '1', '12', '8380212', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIColumn', 'Int', '0', '13', '8380213', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepISpaces', 'Int', '0', '14', '8380214', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIPrint', 'Char', '1', '15', '8380215', 'no', 'no', 'no', "'Y'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIWidthWS', 'Int', '0', '16', '8380216', 'no', 'no', 'no', '0' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepISpan', 'Char', '10', '17', '8380217', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIWrap', 'Char', '1', '18', '8380218', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepISuppress', 'Char', '1', '19', '8380219', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIUnderline', 'Char', '1', '20', '8380220', 'no', 'no', 'no', "'N'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIColExpr', 'VarAlpha', '256', '21', '8380221', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIHeading', 'VarAlpha', '132', '22', '8380222', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIMask', 'VarAlpha', '132', '23', '8380223', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', 'DirName', 'Alpha', '18', '24', '8380224', 'no', 'no', 'no', "'ZIM'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepGKey', 'Char', '23', '25', '8380225', 'yes', 'yes', 'yes', \
       "$concat(_RepItems._RepName,$mask(_RepItems._RepId,'99999'))" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', '_RepIKey', 'Char', '23', '26', '8380226', 'Unique', 'yes', 'yes', \
       "$concat(_RepItems._RepName,$mask(_RepItems._RepIId,'99999'))" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPAddField( vlBreakStatus, \
       '_RepItems', 'TplFsKey', 'Alpha', '36', '27', '8380227', 'yes', 'yes', 'yes', \
       '$concat(_RepItems.TplName,_RepItems.TplFsName)' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   LPCreateEnt( vlBreakStatus, '_RepItems', '8380200','0024' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%% CREATING ROLES %%%%%%%%%%%%%%%%%%%%%%%%%%

   if $Exist$('ZIM','TplS',12) = 0
      plRoleAdd( vlBreakStatus, 'TplS', 'Templates', '', '8385000' )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   if $Exist$('ZIM','TplFs',12) = 0
      plRoleAdd( vlBreakStatus, 'TplFs', 'TemplFields', '', '8385001' )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   if $Exist$('ZIM','_RPTS',12) = 0
      plRoleAdd( vlBreakStatus, '_Rpts', '_Reports', '', '8385002' )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   if $Exist$('ZIM','_RPTG',12) = 0
      plRoleAdd( vlBreakStatus, '_RptG', '_RepGroups', '', '8385003' )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   if $Exist$('ZIM','_RPTI',12) = 0
      plRoleAdd( vlBreakStatus, '_RptI', '_RepItems', '', '8385004' )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   %%%%%%%%%%%%%%%%%%%%% CREATING RELATIONSHIP %%%%%%%%%%%%%%%%%%%

   plRelAdd( vlBreakStatus, \
      'TplSTplFs', 'Templates.TplName=TemplFields.TplName', '8386000' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRelAdd( vlBreakStatus, \
      'TplSReports', 'Templates.TplName=ZIM._Reports.TplName', '8386001' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRelAdd( vlBreakStatus, \
      '_ReportsRepGroups', 'ZIM._Reports._RepName=_RepGroups._RepName', '8386002' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRelAdd( vlBreakStatus, \
      '_ReportsRepItems', 'ZIM._Reports._RepName=_RepItems._RepName', '8386003' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRelAdd( vlBreakStatus, \
      '_RepGroupsRepItems', '_RepGroups._RepGKey=_RepItems._RepGKey', '8386004' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRelAdd( vlBreakStatus, \
      'TplFsRepItems', 'TemplFields.TplFsKey=_RepItems.TplFsKey', '8386005' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%% CREATING CONSTANTS FOR PAGESIZE %%%%%%%%%%%%%%%%%%%

   plAddConsts( vlBreakStatus, 'rep$11x17',  'char', '"2794;4317;0001;0001;0254;0317;0000;0000"' )   
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$Legal',  'char', '"2159;3556;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$Letter', 'char', '"2159;2794;0001;0001;0254;0317;0000;0000"' )   
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$A3',     'char', '"2970;4200;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$A4',     'char', '"2100;2970;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$A5',     'char', '"1480;2100;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$A6',     'char', '"1050;1480;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$B4',     'char', '"2570;3640;0001;0001;0254;0317;0000;0000"' )              
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$B5',     'char', '"1820;2570;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$8x13',   'char', '"2032;3302;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$825x13', 'char', '"2095;3302;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$85x13',  'char', '"2159;3302;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$8K',     'char', '"3900;2700;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

   plAddConsts( vlBreakStatus, 'rep$16K',    'char', '"1950;2700;0001;0001;0254;0317;0000;0000"' )      
   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%================================ pZimBoot3 =====================================
procedure pZIMBoot3(out vlStatus) local(vlBreakStatus,vlResponse)

on BREAK
   plBreakHandler(vlBreakStatus)
   let vlStatus = 'BREAK'
   SET RESTORE
   RETURN
endon

on error
   if $lasterrcode = 3007 % Maximum number of ZIM directories already accessed.
      let <9> = $MessageBox( \
            $concat('The "Directories" setting\0dmust be increased to ', \
               $trim($GetProperty('Session', 'Directories') + 1), '.'), \
            1, 1, 1, 'Access Directory')
      let vlStatus = 'ERROR'
      RETURN
   endif
   if $lasterrcode = 7885 or $lasterrcode = 7886
   endif
endon

   %% User must be logged in as ZIM or super so objects
   %% belong to correct owner
   if $ZUserid <> 0 or $ZGroupId <> 0
      output '*** Error *** You are not logged in as a "super" user (ZIM, super)'
      output '              Object dictionary extensions cannot be done.'
      output '              Login as ZIM or super and try again.'
      let vlStatus = 'ERROR'
      RETURN
   endif

   set errors off
   if $dirpath() not like '%;$DevCen(%'
      access $DevCen
   endif
   if $dirpath() not like '%;$DeployServices(%'
      access $DeployServices
   endif
   set errors on
   SET SAVE
   set membercount off
   set information off
   let vlBreakStatus = $false
   
   if $DCRevision < '3.5'
      plDCv53r10Update(vlBreakStatus)
      plDCv53r31Update(vlBreakStatus)
      plDCv54r11Update(vlBreakStatus)
      plDCv54r21Update(vlBreakStatus)
      plDCv55r11Update(vlBreakStatus)
      plDCv61r10Update(vlBreakStatus)
      plDCv82r31Update(vlBreakStatus)
   endif

   if $DCRevision < '4.0'
      UpdateODBCAndZOM()
   endif
   
   if $DCRevision < '4.1'
      if $exist$( 'ZIM', 'noguiuse', 18 ) = $false
         % This guy was the former holder of the $SN 492.
         if $exist$( 'ZIM', '$ODVersion', 18 ) = $true
@syser
            erase constant $ODVersion from Zim
         endif
         add 1 Consts let ConstName = 'noguiuse' DirName = 'ZIM' ConstType = 'numeric' ConstValue = '157'
         create constant noguiuse in ZIM $sn 492
         delete 1 Consts where ConstName = 'noguiuse' and DirName = 'ZIM'
      endif
   endif
   
   if $DCRevision < '4.2'
      if $zimos = 11
         system $concat("!mkdir ", $DBPath, "\\zomsaving ") invisible remote
      else
         system $concat("mkdir -p ", $DBPath, "/zomsaving ") remote
      endif
   endif

   if $DCRevision < '9.10' and (vZomOperationMode is $null or vZomOperationMode <> "DC") and $DCRevision <> 0.0
      set errors off
      erase index Directories.NullDirName
      erase index EntitySets.NullDirName
      erase index Fields.NullDirName
      erase index Relationships.NullDirName
      erase index Roles.NullDirName
      erase index Documents.NullDirName
      erase index Forms.NullDirName
      erase index Displays.NullDirName
      erase index Menus.NullDirName
      erase index Windows.NullDirName
      erase index Constants.NullDirName
      erase index Variables.NullDirName
      erase index NamedSets.NullDirName
      set errors on
   endif

   if $DCRevision < '9.13' and (vZomOperationMode is $null or vZomOperationMode <> "DC") and $DCRevision <> 0.0
      set errors off
      create index DFS.NullDirName
      create index DFS.NullFormDirName
      create index DFS.ObjectKey
      create index DFS.NullObjectKey
      set errors on
   endif

   if $DCRevision < '9.14' and (vZomOperationMode is $null or vZomOperationMode <> "DC") and $DCRevision <> 0.0
      % We only need to recreate the CHAR constants as the NUMERIC ones are automatically generated.
      zomrecreate +t consts
   endif

   if $DCRevision < '9.20'
   	% Hide these fields until they would be removed in the future or properly used.
   	set FFS.HMargin fieldlistwidth -1
   	set FFS.VMargin fieldlistwidth -1
   	set FFS.Row fieldlistwidth -1
   	set FFS.Col fieldlistwidth -1
   	set FFS.Wdth fieldlistwidth -1
   	set FFS.Hght fieldlistwidth -1
   	set FFS.WdgDragMode fieldlistwidth -1
   	set FFS.WdgDropTarget fieldlistwidth -1
   	set FFS.Fill fieldlistwidth -1
   endif

   if $DCRevision < '9.20' and (vZomOperationMode is $null or vZomOperationMode <> "DC") and $DCRevision <> 0.0
      if $exist$('FormFields','WdgType',5) = 0
         plFieldAdd( vlBreakStatus, 'FormFields', 'WdgType', 'int', '3', '4600', '495', 'no', 'no', 'no', "" )

         % Test whether the database is Unicode.
         add 1 Consts let constname="$TestUnicode$" constvalue="测试数据库是否为Unicode" consttype="Unichar" dirname="ZIM" -> sTestUnicode
         find 1 Consts where constvalue="????????Unicode"
         if $setcount = 1
            plFieldAdd( vlBreakStatus, 'FormFields', 'WdgCSS', 'varchar', '512', '4610', '496', 'no', 'no', 'no', "" )
         else
            plFieldAdd( vlBreakStatus, 'FormFields', 'WdgCSS', 'Univar', '512', '4610', '496', 'no', 'no', 'no', "" )
         endif
         delete sTestUnicode
      endif
      
      change all FormFields let \
         WdgType = {30 where FT = "0", 31 where FT = "1", 20 where FT = "A", 04 where FT = "B", 08 where FT = "C", \
		    12 where FT = "D", 17 where FT = "E", 06 where FT = "F", 11 where FT = "G", 23 where FT = "H", 10 where FT = "I", \
		    22 where FT = "J", 25 where FT = "K", 07 where FT = "L", 05 where FT = "M", 19 where FT = "N", 09 where FT = "O", \
		    01 where FT = "P", 29 where FT = "Q", 21 where FT = "R", 13 where FT = "S", 03 where FT = "T", 14 where FT = "U", \
		    02 where FT = "V", 24 where FT = "W", 28 where FT = "X", 26 where FT = "Y", 27 where FT = "Z", 07}
      change all FormFields where Default = "B^"? let \
         WdgCSS = $concat('{"BGGradient":["', $substring(Default, 3, 2), '","', $substring(Default, 5, 2), '","', $substring(Default, 7, 6), '","', $substring(Default, 13, 6), '"],', \
                          ' "FGGradient":["', $substring(Default, 22, 2), '","', $substring(Default, 24, 2), '","', $substring(Default, 26, 6), '","', $substring(Default, 32, 6), '"],', \
                          ' "Border":["', $substring(Default, 41, 2), '","', $substring(Default, 43, 2), '","', $substring(Default, 45, 6), '"]}') \
         Default = ""
      zomerase +t Forms, Menus
      zomcreate +t Forms, Menus
   endif

   % Put here newer changes using a similar structure as below.
   % "x.xx" is the $CurrentDCRevision, that is, the current state of the product.
   %if $DCRevision < 'x.xx'
   %   ...
   %endif

   % This is the last thing to be done. Don't touch it!
   if $DCRevision < $CurrentDCRevision
      plConstUpdate( vlBreakStatus, '$DCRevision', 'numeric', $CurrentDCRevision, '497' )
   endif

   %% While were here, make sure there is a Directory entry for ZIM
   compute 1 Directories where DirName = 'ZIM' evaluate ('')
   if $membercount = 0
      add 1 Directories \
         let \
            Directories.DirName = 'ZIM' \
            Directories.OwnerDirName = 'ZIM' \
            Directories.DirId = '1' \
            Directories.Foreign = 'no'
   endif

   IF vZomOperationMode is $Null
      ZOMFixup
      ZOMTouch * ;i
   ENDIF

   let vlStatus = 'OK'
   SET RESTORE
endprocedure
%========================== End of Procedure ====================================
