%--------------------------------- UNLOAD --------------------------------------
%
%    Purpose: Unload data from an existing set into a structured document.
%
%    Syntax:  UNLOAD <set name> <output document> [<directory name>]
%
%    Notes:   1) If the parameters are not given you will be prompted.
%             2) <set name> must be a entity set or relationship with fields.
%             3) <output document> - only the first 8 characters are used.
%
%------------------------------------------------------------------------------

on break
	set restore
	release $utils
	if "#<6>" = $true	% trash temporary document
		erase document #<2> from #<3>
		delete documents where docname='#<2>' and dirname="#<7>"
		delete all fields where ownername='#<2>' and dirname="#<7>"
	endif
	return
endon

ZOMEnable

if '#<1>' = '?'
	output
	output 'UNLOAD exports the records from an existing entity set (or'
	output 'relationship with fields) into a structured document. All data'
	output 'exported is converted to character (i.e. readable) format. The'
	output 'data can later be imported into the entity set (or another entity'
	output 'set) using the RELOAD utility program or the ADD command.'
	output
	output "use: UNLOAD [ <entity set>    <output document>  [<directory name>] ]"
	output "            [ <relationship>                                        ]"
	pause
	return
endif


set save
set reset

% Access the ZIM utilities directory, $utils, if it isn't already accessed.

set errors off
set warnings off
let $lasterrcode = 0  $lasterrlevel = 0
let $utils._dirstring = ""
if $lasterrcode = 4008
   let $lasterrcode = 0  $lasterrlevel = 0
   access $utils read
endif
set errors on
set warnings on

% Terminate if unable to access $utils
if $lasterrlevel > 2	% Error accessing $utils...
	output
	output "*** ERROR *** Unable to access ZIM utilities directory, '$Utils'."
	output "*** ERROR *** Check that your ZIM environment variable is set up properly"
	output
	set exception break
endif

set information off
set nullvalue ''
set checksets off
let <6> = $false
output "--------------------------------- UNLOAD ------------------------------------"
output ""
output "    Purpose:"
output "        Unload the data in an existing set to a document."
output "        The output document is created here."
output ""
output "    Steps to Follow:"
output "        1) Unload the data using this user command."
output "        2) Erase the definition of the set from the"
output "           ZIM directory that it was created in."
output "        3) Modify the definition of the Entity Set or"
output "           Relationship with fields in the Data Dictionary,"
output "           (if necessary)."
output "        4) Create the new set in the given directory."
output "        5) Use the RELOAD command to reload the data."
output ""
pause
output ""
output "    Syntax:"
output ""
output "        UNLOAD <set name> <output document> [<directory name>]"
output ""
output "    Notes:"
output ""
output "        1)  unload.mac is used as a temporary document"
output "            to build the output document so make sure it isn't"
output "            in your database directory"
output ""
output "        2)  If the parameters are not given you will"
output "            be prompted for them."
output ""
output "        3)  <set name> must be a entity set or"
output "            relationship with fields."
output ""
output "        4)  <output document> - only the first 8"
output "            characters are used."
output ""
output "------------------------------------------------------------------------------"
pause
output ""

% do this to avoid an error message from Operating System when deleting
set output $utils._$savemacro$
output
set output terminal

output '                    *** UNLOAD ***'
output
output 'To abort at any time simply press ENTER when prompted for information.'
output
output
if '#<1>' = ''	% no ent/rel name given
    output '* Enter the name of the entity set or relationship to be unloaded.'
    output 'Entity Set or Relationship name: ';
    input <1>
    output
    if '#<1>' = ''
        set exception break
    endif
	output '* Enter the name of the ZIM directory in which (#<1>) is defined.'
	output '  (Press return for the default directory "zim".)'
	output 'ZIM Directory name: ';
	input <3>
    output
    if '#<3>' = ''
       let <3> = 'zim'
    endif
else
    let <8> = $true        % Flag to indicate parameters passed on call to UNLOAD.
    if '#<3>' = ''
        let <3> = 'zim'
    endif
endif

    % Data Dictionary search for the entity set or relationship name.
    % Note: ent and rel with same name ==> everything ok but messages.

let <7> = "#<3>"	% will be dirname or blank
find 1 entitysets where entname='#<1>' and dirname='#<3>'
if $setcount = 0
	find 1 entitysets where entname='#<1>' and dirname=''
	let <7> = ""
endif
if $setcount > 0
    let <1> = $trim(EntName)
    let <9> = 'entity set'
else
	let <7> = "#<3>"	% will be dirname or blank
    find 1 relationships where relname='#<1>' and dirname = '#<3>'
	if $setcount = 0
	    find 1 relationships where relname='#<1>' and dirname = ''
		let <7> = ""
	endif
    if $setcount > 0
        let <1> = $trim(RelName)
        let <9> = 'relationship'
    else
        output '+++ UNLOAD error +++ No entity set or relationship definition for (#<1>)'
        output '                     in the Data Dictionary entity sets ENTITYSETS or RELATIONSHIPS.'
        set exception break
    endif
endif

%% set up to create document to hold data

while
    if '#<2>'=''
        output '* Enter the name of a temporary document to be used for storing'
        output '  the #<9> (#<1>) data (max 8 characters).'
        output 'Document name: ';
        input <2>
        output
    endif
    if '#<2>' = ''
		set exception break
    endif
    let <2> = $substring ('#<2>', 1, 8)
            % Check if docname in-use.
    if $iszimname ('#<2>') <> $true
        output 'The document name ("#<2>") must be a valid ZIM name.'
        output "    Please enter the document name again."
        output
        let <2> = ""
        continue
    endif
    find 1 docs where docname='#<2>' and dirname = "#<7>"
    if $setcount = 0
        find 1 fields where ownername='#<2>' and dirname = "#<7>"
    endif
    if $setcount > 0
        output '*** UNLOAD Warning *** The Document name (#<2>) is in-use.'
        let <2> = ''
		continue
    endif
	break
endwhile

    % Data Dictionary search for the fields of the entity set or relationship.
find fields where ownername='#<1>' and dirname = "#<7>" and virtual <> "yes" -> fieldset
if $setcount = 0
    output '+++ UNLOAD error +++ No field descriptions for (#<1>) in the'
    output '                     Data Dictionary set FIELDS.'
    if '#<9>' = 'relationship'
    output '                     You can only UNLOAD relationships with fields.'
    endif
    set exception break
endif
let <4> = $concat ($trim('#<2>'), '.dat', '')

output
output '---------------------------- UNLOAD ---------------------------------'
output $tochar('#<9>',18) $tochar('directory',18) $tochar('document',18) 'file'
output $tochar('#<1>',18) $tochar('#<3>',18)        $tochar('#<2>',18)       '#<4>'
output

    % Create the temporary document.
output '* Now creating the temporary document (#<2>) - file name (#<4>).'
let <6> = $true	% signal that temp doc has been defined
add fields from fieldset let ownername='#<2>' objectkey = $null -> fieldset2

change all fieldset2 where ownername='#<2>' \
    let type =   {'numeric' where type in ('int','longint',    \
                               'vastint','date')       \
                 ,'alpha'   where type='varalpha' or type='varchar' or type='univar'     \
                 ,type                                                 \
                 }                                                     \
        length = { 8 where type='int' or type='date'                   \
                 ,12 where type='longint'                              \
                 ,17 where type='vastint'                              \
                 ,length                                               \
                 }
add documents let docname='#<2>' filename='#<4>' dirname="#<7>"
create document #<2> in #<3>
output
output '* Temporary document (#<2>) created with'
output '  fields of type Int, LongInt, VastInt and Date replaced by Numeric, and'
output '  fields of type VarAlpha, UniVar and VarChar replaced by Alpha.'
output
output 'Does everything seem OK to start unloading? ( i.e. no error messages? )'
output 'Enter y/n:';
input <5>
if $tolower('#<5>') <> 'y'
    set exception break
endif
        % UNLOADing
output
output 'Unloading (#<1>...)'
output
set headings off
set column 0

% construct the list command for all fields except virtual fields
set output $utils._$savemacro$
set membercount off
output "SET SAVE"
output "SET EOLNTRANSLATE $Utils.cEOLNTRANSLATE"
output "list all #<3>.#<1> format \\"
list all fields where ownername='#<1>' and dirname = "#<7>" and virtual <> "yes" \
         sorted by sn \
         format $concat ("$value (", $trim (ownername), ".", $trim (fieldname), " , \"\") \\")
output % necessary to terminate last continuation
output "SET RESTORE"

% dump the data!
set output #<3>.#<2>
_$savemacro$
        % Check for Errors or Warnings during UNLOADing.
if $errcode <> 0 and $errlevel > 1    % Warning or Error has occurred.
    set output terminal
    output '+++ UNLOAD +++ An error or warning has been detected.'
    output '    Would you like the temporary document (#<2>) removed?'
    output 'Enter y/n:';
    input <5>
    if $tolower('#<5>') = 'n'
		let <6> = $false
    endif
	set exception break
endif

% Successful UNLOAD.
set output terminal
output '* The data in #<9> (#<1>) has been unloaded into'
output '  the document (#<2>) with filename (#<4>).'
output
output '* To re-create the #<9>:'
output '    1) ERASE the #<9> (#<1>) from directory (#<3>)'
output '    2) Modify the definition of (#<1>) in the Data Dictionary,'
output '       (if necessary).'
output '    3) CREATE the new #<9> (#<1>).'
output '    4) Use RELOAD to reload the data.'

% get rid of temporary macro file
set output $utils._$savemacro$
set out terminal
let <6> = $false	%% don't remove the temp doc definition
set exception break
