%%%%             Exporting tool to convert ZIMxx databases to ZIM8.
%%%%             =================================================
%%%%     Objects will be exported in the Z50 format compatible with version 8.
%%%%     For version 4.x users: Databases must be converted to Zim 7.x before converting to Zim 8.
%%%%
%%%%     To run this program you need to be in SINGLE USER since it creates and erases
%%%%        objects into/from the Zim data dictionary.
%%%%     All objects created/erased will be owned by the ZIM directory and
%%%%        will be destroyed at the end of the process.
%%%%      BEFORE executing this process a ZOMRESET should be executed to fix all
%%%%        possible erros on the Objectlist table.
%%%%     This process assumes that the database data dictionary is correct
%%%%        and all objects in error were fixed.
%%%%
%%%%   Celso Bressan, 24/06/2013: Modernizing and making it to work.
%%%%   Livio Taufer,  25/09/2017: Adapted for use ZOM  facilities and
%%%%       make all adaptations here and not some in CNV1 and others in CNV2
%%%%       this way zim8cnv2 will be a simple zomimport
%%%
%%%  UPDATEs:  11-Oct-17 - Several modifications from original CNV1 and CNV2
%%%            07-Nov-17 - Blank formfield Names and negative wdgRow and wdgCol
%%%            10-Nov-17 - Exporting the tabcontrol data just on case
%%%            15-Nov-17 - Report UNKNOWN Ent and Reltypes (not ZIM, JDBCSAM,"")
%%%            15-Nov-17 - Test if there are TABBED FORM and report
%%%            15-Nov-17 - Create a document to report issues during migration
%%%            15-Jan-18 - Listbox (not dynamic listbox) transformed to Combobox
%%%            26-Feb-18 - Included the check names ambiguity (same name in different
%%%                        objects used in relationships and roles conditions)
%%%            27-Feb-18 - Create document to hold the migtration report in CNV1.
%%%                        DocName=$$$CNV1Report$$$ e filename=./MIGRATION/REPORTS/cnv1Report.txt
%%%            27-Feb-18 - Create folders MIGRATION78/REPORTS if they do not exist
%%%            06-Mar-18 - Adapt FFS WdgOnImage containing *.BMP to blank
%%%            27-Mar-18 - Fixed procedure plChecKAmbiguity using $Toalpha
%%%
%%%
% =======================================================================================================
LocalProcedure plSetOut(vlDocument)
%
   set SAVE
   set nullvalue '$null'
   set headings off
   set output fieldlistwidth off
   set output maskfield off
   set output virtualfield off
   set output trim on
   set output format commadelimited
   set delimiter ':'
   set textdelimiter '^'
   set eolntranslate $Utils.cEOLNTRANSLATE
   let vZOMCommandLine = $concat("set output ",$trim(vlDocument))
   pExecuteMacro(vDc1,vZOMCommandLine)

EndProcedure      %%%   plSetOut
%=================================================================================================================

LocalProcedure plExpMetaObjs()

%---------------------------------------------------------------------------------------------------
%%% Meta Objects.
output '    Exporting all MetaObjects.'
set Save
plSetOut("_ReportDoc")
list all _Reports
plSetout("_RepColumnDoc")
list all _RepColumns
plSetout("_QSortFormatDoc")
list all _QSortFormat
plSetout("_QSelectDoc")
list all _QSelect
plSetout("_ComponentDoc")
list all  _Components
plSetout("ObjectActionDoc")
list all ObjectAction
plSetout("EventActionDoc")
list all EventAction
plSetout("ApplMessageDoc")
list all ApplMessages
plSetout("ApplMessTitleDoc")
list all ApplMessTitles
plSetout("RemoteProcDoc")
list all RemoteProcedures
plSetout("ResultSetDoc")
list all ResultSets
plSetout("ResultFieldDoc")
list all ResultFields
plSetout("TabControlDoc")
list all TabControl
set restore

endprocedure      %%% plExpMetaObjs
%=================================================================================================================
LocalProcedure plClearZ50()
%
set output dcz50Ents
set output dcz50Rels
set output dcz50Docs
set output dcz50Consts
set output dcz50Descs
set output dcz50DFS
set output dcz50Dirs
set output dcz50Disps
set output dcz50Docs
set output dcz50FFS
set output dcz50Flds
set output dcz50Forms
set output dcz50Keywords
set output dcz50Menus
set output dcz50Objlist
set output dcz50PtrGrps
set output dcz50Rels
set output dcz50Roles
set output dcz50Sets
set output dcz50Users
set output dcz50Vars
set output dcz50wins
set output terminal

EndProcedure      %%%   plClearZ50
%=================================================================================================================
localprocedure plExportShConsts()

   plSetOut("dcZ50Consts")
   list all ShConstants \
      format \
         ShConstants.ConstName \
         ShConstants.ConstValue \
         ShConstants.ConstType \
         ShConstants.DirName \
         ShConstants.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShDescs()

   plSetOut("dcZ50Descs")
   list all ShDDDescriptions \
      format \
         ShDDDescriptions.DDObjectDesc \
         ShDDDescriptions.DDObjectName \
         ShDDDescriptions.DDObjectType \
         ShDDDescriptions.DDOwnerName \
         ShDDDescriptions.DirName \
         SHDDDescriptions.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShDfs()

   plSetOut("dcZ50Dfs")
   list all ShDisplayForms \
      format \
         ShDisplayForms.DisplayName \
         ShDisplayForms.FormName \
         ShDisplayForms.FormNum \
         ShDisplayForms.FSN \
         ShDisplayForms.Row \
         ShDisplayForms.Col \
         ShDisplayForms.LoadOrder \
         ShDisplayForms.RD \
         ShDisplayForms.RA \
         ShDisplayForms.FormDirName \
         ShDisplayForms.DirName \
         ShDisplayForms.WdgRow \
         ShDisplayForms.WdgCol \
         ShDisplayForms.HMargin \
         ShDisplayForms.VMargin \
         ShDisplayForms.WdgHMargin \
         ShDisplayForms.WdgVMargin \
         ShDisplayForms.WdgWdth \
         ShDisplayForms.WdgHght \
         ShDisplayForms.Wdth \
         ShDisplayForms.Hght \
         ShDisplayForms.WdgZOrder \
         ShDisplayForms.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================
localprocedure plExportShDirs()

   plSetOut("dcZ50Dirs")
   list all ShDirectories \
      WHERE DIRNAME <> 'ZIM' \
      format \
         ShDirectories.DirName \
         ShDirectories.OwnerDirName \
         ShDirectories.DirId \
         ShDirectories.Foreign \
         ShDirectories.ForeignAccess \
         ShDirectories.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShDisps()

   plSetOut("dcZ50Disps")
   list all ShDisplays \
      format \
         ShDisplays.DisplayName \
         ShDisplays.DirName \
         ShDisplays.WdgTag \
         ShDisplays.HMargin \
         ShDisplays.VMargin \
         ShDisplays.WdgHMargin \
         ShDisplays.WdgVMargin \
         ShDisplays.WdgWdth \
         ShDisplays.WdgHght \
         ShDisplays.Wdth \
         ShDisplays.Hght \
         ShDisplays.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShDocs()

   plSetOut("dcZ50Docs")
   list all ShDocuments \
      where docname not in ("Terminal","Halt","Stdin","stdout","stderr") \
      format \
    ShDocuments.DocName \
    ShDocuments.FileName \
    ShDocuments.DocType \
    ShDocuments.AvgSize \
    ShDocuments.DirName \
    ShDocuments.ObjectKey
%%%    ShDocuments.DocPeruser %% Zim 8.0
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShEnts()

   plSetOut("dcZ50Ents")
   list all ShEntitySets \
      format \
         ShEntitySets.EntName \
         ShEntitySets.EntType \
         ShEntitySets.AvgSize \
         ShEntitySets.DirName \
         ShEntitySets.ObjectKey \
         ShEntitySets.RemoteName \
         ShEntitySets.RemoteOwnerName
%%%         ShEntitySets.EntPerUser %% Zim 8.0
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShFfs()

   plSetOut("dcZ50Ffs")
   list all ShFormFields \
      format \
         ShFormFields.FormName \
         ShFormFields.FieldName \
         ShFormFields.FNum \
         ShFormFields.FFSN \
         ShFormFields.Row \
         ShFormFields.Col \
         ShFormFields.Wdth \
         ShFormFields.Hght \
         ShFormFields.FieldLen \
         ShFormFields.FT \
         ShFormFields.DT \
         ShFormFields.PM \
         ShFormFields.DM \
         ShFormFields.BC \
         ShFormFields.FC \
         ShFormFields.IT \
         ShFormFields.TE \
         ShFormFields.VR \
         ShFormFields.SK \
         ShFormFields.AC \
         ShFormFields.Case \
         ShFormFields.Just \
         ShFormFields.Fill \
         ShFormFields.MinLen \
         ShFormFields.AN \
         ShFormFields.CM \
         ShFormFields.VA \
         ShFormFields.DataMask \
         ShFormFields.ValRule \
         ShFormFields.Default \
         ShFormFields.HelpMsg \
         ShFormFields.ErrorMsg \
         ShFormFields.DirName \
         ShFormFields.WdgTag \
         ShFormFields.WdgId \
         ShFormFields.WdgParId \
         ShFormFields.WdgSubClass \
         ShFormFields.WdgStyle \
         ShFormFields.WdgRow \
         ShFormFields.WdgCol \
         ShFormFields.WdgWdth \
         ShFormFields.WdgHght \
         ShFormFields.WdgOrientation \
         ShFormFields.WdgFCRed \
         ShFormFields.WdgFCGreen \
         ShFormFields.WdgFCBlue \
         ShFormFields.WdgBCRed \
         ShFormFields.WdgBCGreen \
         ShFormFields.WdgBCBlue \
         ShFormFields.WdgBorderStyle \
         ShFormFields.WdgBackground \
         ShFormFields.WdgShadowSize \
         ShFormFields.WdgLabel \
         ShFormFields.WdgOnImage \
         ShFormFields.WdgOffImage \
         ShFormFields.WdgNullImage \
         ShFormFields.WdgFont \
         ShFormFields.WdgFontSize \
         ShFormFields.WdgPointerStyle \
         ShFormFields.WdgMultiLine \
         ShFormFields.WdgAutoSize \
         ShFormFields.WdgDragIcon \
         ShFormFields.WdgOnValue \
         ShFormFields.WdgOffValue \
         ShFormFields.WdgNullValue \
         ShFormFields.WdgMinValue \
         ShFormFields.WdgMaxValue \
         ShFormFields.WdgLargeIncrement \
         ShFormFields.WdgSmallIncrement \
         ShFormFields.WdgZOrder \
         ShFormFields.WdgTabStop \
         ShFormFields.WdgAutoSelect \
         ShFormFields.WdgScrollBars \
         ShFormFields.WdgEditOptions \
         ShFormFields.WdgNumVisibleLines \
         ShFormFields.WdgIsDefault \
         ShFormFields.WdgDragMode \
         ShFormFields.WdgDropTarget \
         ShFormFields.WdgModified \
         ShFormFields.WdgClick \
         ShFormFields.WdgDblClick \
         ShFormFields.WdgAutoDrop \
         ShFormFields.WdgLabelPosition \
         ShFormFields.WdgNumVisibleCols \
         ShFormFields.HMargin \
         ShFormFields.VMargin \
         ShFormFields.WdgHMargin \
         ShFormFields.WdgVMargin \
         ShFormFields.WdgInputMask \
         ShFormFields.WdgItems \
         ShFormFields.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShFlds()

   plSetOut("dcZ50Flds")
   list all ShFields \
      format \
         ShFields.SN \
         ShFields.FieldName \
         ShFields.OwnerName \
         ShFields.Type \
         ShFields.Length \
         ShFields.Decimals \
         ShFields.Reqd \
         ShFields.Index \
         ShFields.Virtual \
         ShFields.ListWidth \
         ShFields.FieldHeading \
         ShFields.FieldMask \
         ShFields.Default \
         ShFields.EQSel \
         ShFields.BWSel \
         ShFields.LGSel \
         ShFields.DirName \
         ShFields.ObjectKey \
         ShFields.RemoteName
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShForms()

   plSetOut("dcZ50Forms")
   list all ShForms \
      format \
         ShForms.FormName \
         ShForms.DirName \
         ShForms.Wdth \
         ShForms.Hght \
         ShForms.WdgWdth \
         ShForms.WdgHght \
         ShForms.HMargin \
         ShForms.VMargin \
         ShForms.WdgHMargin \
         ShForms.WdgVMargin \
         ShForms.WdgTag \
         ShForms.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShKeywords()

   plSetOut("dcZ50Keywords")
   list all ShKeywords \
      format \
         ShKeywords.Keyword \
         ShKeywords.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShMenus()

   plSetOut("dcZ50Menus")
   list all ShMenus \
      format \
         ShMenus.MenuName \
         ShMenus.DirName \
         ShMenus.MenuNum \
         ShMenus.WdgTag \
         ShMenus.MenuType \
         ShMenus.WdgClass \
         ShMenus.WdgSubClass \
         ShMenus.WdgStyle \
         ShMenus.Row \
         ShMenus.Col \
         ShMenus.WdgRow \
         ShMenus.WdgCol \
         ShMenus.DM \
         ShMenus.FC \
         ShMenus.WdgFCRed \
         ShMenus.WdgFCGreen \
         ShMenus.WdgFCBlue \
         ShMenus.BC \
         ShMenus.WdgBCRed \
         ShMenus.WdgBCGreen \
         ShMenus.WdgBCBlue \
         ShMenus.WdgBackground \
         ShMenus.WdgBorderStyle \
         ShMenus.WdgShadowSize \
         ShMenus.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShObjList()

   plSetOut("dcZ50ObjList")
   list all ShObjectList \
      format \
         ShObjectList.ObjectName \
         ShObjectList.DirName \
         ShObjectList.OwnerName \
         ShObjectList.ObjectType \
         ShObjectList.ObjectKey \
         ShObjectList.NotExist \
         ShObjectList.ToProcess \
         ShObjectList.DependencyLevel \
         ShObjectList.ProcessErrors \
         ShObjectList.NotDefined \
         ShObjectList.NotActive \
         ShObjectList.CompileAble \
         ShObjectList.CompileError \
         ShObjectList.CompileStatus \
         ShObjectList.Locked \
         ShObjectList.AutoDataSave \
         ShObjectList.DataSaveAble \
         ShObjectList.Selected \
         ShObjectList.ReNamed \
         ShObjectList.Changed \
         ShObjectList.Moved \
         ShObjectList.NewObject \
         ShObjectList.FileNumber \
         ShObjectList.SysSeqNo \
         ShObjectList.TmpCounter1 \
         ShObjectList.TmpCounter2 \
         ShObjectList.TmpCounter3 \

   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShPtrGrps()

   plSetOut("dcZ50PtrGrps")
   list all ShPainterGroups \
      format \
         ShPainterGroups.WinName \
         ShPainterGroups.WinDirName \
         ShPainterGroups.FDName \
         ShPainterGroups.FDDirName \
         ShPainterGroups.MenuName \
         ShPainterGroups.MenuDirName
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShRels()

   plSetOut("dcZ50Rels")
   list all ShRelationships \
      format \
         ShRelationships.RelName \
         ShRelationships.RelCondition \
         ShRelationships.RelType \
         ShRelationships.AvgSize \
         ShRelationships.DirName \
         ShRelationships.ObjectKey \
         ShRelationships.RemoteName \
         ShRelationships.RemoteOwnerName
%%%         ShRelationships.RelPerUser %% Zim 8.0
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShRoles()

   plSetOut("dcZ50Roles")
   list all ShRoles \
      format \
         ShRoles.RoleName \
         ShRoles.OwnerName \
         ShRoles.RoleCondition \
         ShRoles.DirName \
         ShRoles.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShSets()

   plSetOut("dcZ50Sets")
   list all ShNamedSets \
      format \
         ShNamedSets.SetName \
         ShNamedSets.SetSpec \
         ShNamedSets.AvgSize \
         ShNamedSets.DirName \
         ShNamedSets.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShUsers()

   plSetOut("dcZ50Users")
   list all ShUsers \
      format \
         ShUsers.GroupId \
         ShUsers.UserId \
         ShUsers.UserName
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShVars()

   plSetOut("dcZ50Vars")
   list all ShVariables \
      format \
         ShVariables.VarName \
         ShVariables.Type \
         ShVariables.Length \
         ShVariables.Decimals \
         ShVariables.Size \
         ShVariables.DirName \
         ShVariables.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportShWins()

   plSetOut("dcZ50Wins")
   list all ShWindows \
      format \
         ShWindows.WinName \
         ShWindows.WNum \
         ShWindows.Row \
         ShWindows.Col \
         ShWindows.Wdth \
         ShWindows.Hght \
         ShWindows.DM \
         ShWindows.BC \
         ShWindows.FC \
         ShWindows.Fill \
         ShWindows.Bord \
         ShWindows.BordDM \
         ShWindows.BordBC \
         ShWindows.BordFC \
         ShWindows.BordDefault \
         ShWindows.DirName \
         ShWindows.WinType \
         ShWindows.WdgBorderStyle \
         ShWindows.WdgCaption \
         ShWindows.WdgResize \
         ShWindows.WdgMinimize \
         ShWindows.WdgMaximize \
         ShWindows.WdgSysMenu \
         ShWindows.WdgPSWdth \
         ShWindows.WdgPSHght \
         ShWindows.WdgPSRow \
         ShWindows.WdgPSCol \
         ShWindows.WdgModal \
         ShWindows.WdgScrollBars \
         ShWindows.WdgClass \
         ShWindows.WdgSubClass \
         ShWindows.WdgRow \
         ShWindows.WdgCol \
         ShWindows.WdgWdth \
         ShWindows.WdgHght \
         ShWindows.WdgFontSize \
         ShWindows.WinOwnerName \
         ShWindows.WinOwnerDirName \
         ShWindows.WdgAlwaysOnTop \
         ShWindows.WdgFCRed \
         ShWindows.WdgFCGreen \
         ShWindows.WdgFCBlue \
         ShWindows.WdgBCRed \
         ShWindows.WdgBCGreen \
         ShWindows.WdgBCBlue \
         ShWindows.WdgLostFocus \
         ShWindows.WdgGotFocus \
         ShWindows.WdgClosed \
         ShWindows.WdgHScrolled \
         ShWindows.WdgVScrolled \
         ShWindows.WdgMaximized \
         ShWindows.WdgMinimized \
         ShWindows.WdgMoved \
         ShWindows.WdgResized \
         ShWindows.WdgRestored \
         ShWindows.WdgFont \
         ShWindows.WdgLabel \
         ShWindows.WdgIcon \
         ShWindows.WdgMove \
         ShWindows.WdgAutoSize \
         ShWindows.HMargin \
         ShWindows.VMargin \
         ShWindows.WdgHMargin \
         ShWindows.WdgVMargin \
         ShWindows.WdgAvailable \
         ShWindows.WdgTag \
         ShWindows.WdgStyle \
         ShWindows.ObjectKey
   SET RESTORE

endprocedure
%=================================================================================================================

localprocedure plExportSD()   %%% Based on pZOMExportSD to run on 7.xx

   output "    Exporting Shadow Dictionary to .Z50 Files"
   pZOMSetObjVars('NULL')
   output "    Exporting Shadow ObjectList"
   plExportShObjList()

   output "    Exporting Shadow KeyWords"
   plExportShKeywords()

   output "    Exporting Shadow Ents"
   plExportShEnts()

   output "    Exporting Shadow Rels"
   plExportShRels()

   output "    Exporting Shadow Docs"
   plExportShDocs()

   output "    Exporting Shadow Fields"
   plExportShFlds()

   output "    Exporting Shadow Roles"
   plExportShRoles()

   output "    Exporting Shadow Forms"
   plExportShForms()

   output "    Exporting Shadow Menus"
   plExportShMenus()

   output "    Exporting Shadow Ffs"
   plExportShFfs()

   output "    Exporting Shadow Disps"
   plExportShDisps()

   output "    Exporting Shadow Dfs"
   plExportShDfs()

   output "    Exporting Shadow PtrGrps"
   plExportShPtrGrps()

   output "    Exporting Shadow Sets"
   plExportShSets()

   output "    Exporting Shadow Wins"
   plExportShWins()

   output "    Exporting Shadow Consts"
   plExportShConsts()

   output "    Exporting Shadow Dirs"
   plExportShDirs()

   output "    Exporting Shadow Vars"
   plExportShVars()

   output "    Exporting Shadow Users"
   plExportShUsers()

   output "    Exporting Shadow Descs"
   plExportShDescs()

endprocedure
%=================================================================================================================
Localprocedure plDocAdd(vlDocName, vlFileName, vlDocType)
%%%      Adds and create a temporary document
   let <DocName> = $trim( vlDocName ) \
       <FileName>= $trim( vlFileName )

   if $exist$( 'ZIM', '#<DocName>', 9 ) = $FALSE
      add 1 Docs \
          let \
              Docs.DocName = '#<DocName>'   \
              Docs.FileName = '#<FileName>'  \
              Docs.DocType = $trim(vlDocType)  \
              Docs.DirName = 'ZIM'     \
          -> $$$sDoc$$$

      zomcreate  #<DocName>
%%%      delete $$$sDoc$$$
   endif
endprocedure
%=================================================================================================================

Localprocedure plFieldAdd(vlOwnerNm, vlFldName, vlDataType, vlLength, vlSN)

   let <SN>      = $trim( vlSN ) \
       <Seq>     = $trim( vlSeq ) \
       <OwnerNm> = $trim( vlOwnerNm ) \
       <FldName> = $trim( vlFldName ) \
       <DataType>= $trim( vlDataType ) \
       <Length>  = $trim( vlLength )

   find Zim.Fields wh Fields.Ownername = $trim( vlOwnerNm ) and   \
                      Fields.FieldName = $trim( vlFldName ) and   \
                      DirName = 'Zim'
   if $setcount = 0
      add 1 Zim.Fields \
          let \
            Fields.SN           = $trim( vlSN ) \
            Fields.FieldName    = $trim( vlFldName ) \
            Fields.OwnerName    = $trim( vlOwnerNm ) \
            Fields.Type         = $trim( vlDataType ) \
            Fields.Length       = $trim( vlLength ) \
            Fields.Decimals     = '0' \
            Fields.Reqd         = 'No' \
            Fields.Index        = 'No' \
            Fields.Virtual      = 'No' \
            Fields.ListWidth    = 0 \
            Fields.FieldHeading = $null \
            Fields.FieldMask    = $null \
            Fields.Default      = '' \
            Fields.EQSel        = '1.000' \
            Fields.BWSel        = '1.000' \
            Fields.LGSel        = '1.000' \
            Fields.DirName      = 'ZIM' \
         -> $$$sFld$$$
    endif
 Endprocedure

% =================================================================================================
LocalProcedure CreateObjs()
%
%     Create all necessary documents and folder
%        Creating folder PORT. If it may already exist.

%%%      Clean up directory port
   system $concat("!mkdir ", $DBPath, "\\port") invisible remote
   system "!del EXISTEJDBCSAM" invisible remote
   system "!del /Q port\\*.*" invisible remote
   system $concat("!mkdir ", $DBPath, "\\MIGRATION78\\REPORTS") invisible remote

   find 1 docs wh docname='$$$Areas$$$'
   if $setcount=0
      plFieldAdd("$$$Areas$$$", "Text", "ALPHA", 100, 1)
      plDocAdd("$$$Areas$$$", "areas.zim", "DATA")
   endif

   find 1 docs wh docname='$$$z50ents$$$'
   if $setcount=0
      plFieldAdd("$$$z50ents$$$", "Text", "ALPHA", 100, 1)
      plDocAdd("$$$z50ents$$$", "./port/ents.z50", "DATA")
   endif

   find 1 docs wh docname='$$$z50rels$$$'
   if $setcount=0
      plFieldAdd("$$$z50rels$$$", "Text", "ALPHA", 500, 1)
      plDocAdd("$$$z50rels$$$", "./port/rels.z50", "DATA")
   endif

   find 1 docs wh docname='$$$z50docs$$$'
   if $setcount=0
      plFieldAdd("$$$z50Docs$$$", "Text", "ALPHA", 100, 1)
      plDocAdd("$$$z50Docs$$$", "./port/docs.z50", "DATA")
   endif

   find 1 docs wh docname='$$$CNV1Report$$$'
   if $setcount=0
      plFieldAdd("$$$CNV1Report$$$", "Text", "ALPHA", 100, 1)
      plDocAdd("$$$CNV1Report$$$", "./MIGRATION78/REPORTS/CNV1Report.txt", "DATA")
   endif

endprocedure %%%   CreateObjs

% =================================================================================================
localprocedure plCheckAmbiguity() local(lMessage,lWarning)
%              CHECK RELATIONSHIPS AND ROLES POSSIBLE AMBIGUITY
%     Verify the occurrence of fields and FFS names that conflicts with object names
%     and are used in reltionship and role conditions.
%     The affected relcondition is modified to qualify the ambiguous name with its directory name.
%     The modified relationships and roles are reported in ZOMOutputDoc
%     The
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set save
set headings off
set out trim on
%
%        The Entityset ObjMgmtDriverEnt will be used as work table
%          (this table is only used when importing via zimfiles)
delete all ObjMgmtDriverEnt
add ObjMgmtDriverEnt from ents    let AttrName = EntName      ObjType = "Entityset" ObjDir = dirname
add ObjMgmtDriverEnt from dirs    let AttrName = Dirs.Dirname ObjType = "Directory" ObjDir = dirname
add ObjMgmtDriverEnt from docs    let AttrName = DocName      ObjType = "Document"  ObjDir = dirname
add ObjMgmtDriverEnt from roles   let AttrName = Rolename     ObjType = "Role"      ObjDir = dirname
add ObjMgmtDriverEnt from sets    let AttrName = SetName      ObjType = "Namedset"  ObjDir = dirname
add ObjMgmtDriverEnt from disps   let AttrName = DisplayName  ObjType = "Display"   ObjDir = dirname
add ObjMgmtDriverEnt from Forms   let AttrName = FormName     ObjType = "Form"      ObjDir = dirname
add ObjMgmtDriverEnt from menus   let AttrName = MenuName     ObjType = "Menu"      ObjDir = dirname
add ObjMgmtDriverEnt from wins    let AttrName = WinName      ObjType = "Window"    ObjDir = dirname
add ObjMgmtDriverEnt from consts  let AttrName = ConstName    ObjType = "Constant"  ObjDir = dirname
add ObjMgmtDriverEnt from vars    let AttrName = VarName      ObjType = "Variable"  ObjDir = dirname

%   Look for fieldnames that conflict with object names (ents, rels, docs, etc)
set out ZOMTemp
select shfields.fieldname, ObjMgmtDriverEnt.ObjType \
   from ObjMgmtDriverEnt, shFields                         \
   where shfields.Fieldname = ObjMgmtDriverEnt.AttrName    \
   order by shfields.fieldname -> sObjMshFields
find sObjMshFields where $distinct(shFields.Fieldname) is not null -> sObjMshFields

%   Look for FFS fieldnames that conflict with object names (ents, rels, docs, etc)
select shFormFields.fieldname, ObjMgmtDriverEnt.ObjType \
   from ObjMgmtDriverEnt, shFormFields  \
   where shformfields.Fieldname = ObjMgmtDriverEnt.AttrName \
   order by shFormFields.fieldname -> sObjmshFFS
set out terminal
find sObjMshFFS where $distinct(shFormFields.Fieldname) is not null -> sObjMshFFs

set out $$$CNV1Report$$$ append
out $center($tochar("FIELDS AND FORMFIELDS THAT MAY RAISE AMBIGUITY",100))
out
list all sObjmshFields format "Field    ", shFields.fieldname,"may be ambigous with "   \
       $tochar(ObjMgmtDriverEnt.ObjType,9), $tochar(ObjMgmtDriverEnt.AttrName,18)      \
       "in directory" ObjMgmtDriverEnt.ObjDir

list all sObjmshFFS format "FormField", shFormFields.fieldname,"may be ambigous with "   \
       $tochar(ObjMgmtDriverEnt.ObjType,9), $tochar(ObjMgmtDriverEnt.AttrName,18)      \
       "in directory" ObjMgmtDriverEnt.ObjDir
set output terminal

%         Put all conflict names together
find sObjMshFields keep ObjMgmtDriverEnt -> sObjM1
find sObjMshFFS    keep ObjMgmtDriverEnt -> sObjM2
find sObjm1 union sObjm2 -> sambiguous
let vzomInt1 = $setcount

set output $$$CNV1Report$$$ append
out
out $CENTER($TOCHAR("MODIFIED RELATIONSHIPS TO CLEAR POSSIBLE AMBIGUITIES",120))
out $CENTER($TOCHAR("====================================================",120))
set output terminal
let vzomint2 = 0
%        Search for relconditions that use these names and qualify the found names
while (let vzomint1 = vzomint1 - 1) >= 0
   set current  sambiguous
   let <EName> = $trim(ObjMgmtDriverEnt.AttrName)
   let <DName> = $trim(ObjMgmtDriverEnt.ObjDir)
%   out "Pesquisando #<Ename> #<DName>"
   find shRels wh $toalpha(relcondition,256) like "#<EName>.%"   \
               or $toalpha(relcondition,256) like "% #<EName>.%" \
               or $toalpha(relcondition,256) like "%=#<EName>.%" \
               or $toalpha(relcondition,256) like "%(#<EName>.%" \
        -> sSHRels
   if $setcount > 0
      change all sSHRels let Relcondition = $concat(" ",Relcondition)
      Change all sSHRels let Relcondition = $trim($translate($translate($translate(    \
                 $toalpha(relcondition,256)," #<EName>."," #<DName>.#<EName>."), \
                                            "=#<EName>.","=#<DName>.#<EName>."), \
                                            "(#<EName>.","(#<DName>.#<EName>."))
      set output $$$CNV1Report$$$ append
      list all sSHRels format relname dirname relcondition
      let vzomInt2 = vzomint2 + $membercount
      set output terminal
   endif
   next sambiguous
endwhile
if vZOMint2 = 0
   out $CENTER($TOCHAR("\0aNO AMBIGUITY FOUND WITHIN RELATIONSHIPS",120))
endif

%        Search for relconditions that use these names and qualify the found names
top sambiguous
let vzomInt1 = $lastmember(sambiguous)
set output $$$CNV1Report$$$ append
out
out $CENTER($TOCHAR("MODIFIED ROLES TO CLEAR THE EXISTING AMBIGUITY",120))
out
set output terminal
let vzomint2 = 0
while (let vzomint1 = vzomint1 - 1) >= 0
   set current  sambiguous
   let <EName> = $trim(ObjMgmtDriverEnt.AttrName)
   let <DName> = $trim(ObjMgmtDriverEnt.ObjDir)
   out "Pesquisando Roles #<Ename> #<DName>"
   find shRoles wh $toalpha(rolecondition,256) like "#<EName>.%"    \
                or $toalpha(rolecondition,256) like "% #<EName>.%"  \
                or $toalpha(rolecondition,256) like "%=#<EName>.%"  \
                or $toalpha(rolecondition,256) like "%(#<EName>.%"  \
        -> sSHRoles
   if $setcount > 0
      change all sSHRoles let Rolecondition = $concat(" ",Rolecondition)
      Change all sSHRoles let Rolecondition = $trim($translate($translate($translate( \
                   $toalpha(Rolecondition,256)," #<EName>."," #<DName>.#<EName>."),   \
                                 "=#<EName>.","=#<DName>.#<EName>."),                 \
                                 "(#<EName>.","(#<DName>.#<EName>."))
      set output $$$CNV1Report$$$ append
      list all sSHRoles format Rolename dirname Rolecondition
      let vzomint2 = vzomint2 + $membercount
      set output terminal
   endif
   next sambiguous
endwhile
if vZOMint2 = 0
   set output $$$CNV1Report$$$ append
   out $CENTER($TOCHAR("NO AMBIGUITY FOUND WITHIN ROLES FIELDS AND FFS",120))
   set output terminal
endif
set restore

EndProcedure      %  plCheckAmbiguity

% =================================================================================================
%%%
%%%                     MAIN PROGRAM
%%%
SET SAVE
set information off
set membercount off
SET EXECUTEMODE ZIMMODE
SET OUTPUT TRIM ON
zomenable
let vzomerrorlog = $False     %%%%  Disable the screen ZOM log
let vZOMDisplayActive = $FALSE
let vZOMDisplay = 'N'
%%%         vDPSBreak identifica se dve ser executado ou nao um BYE ao final da execucao
if $toupper("#<1>") = "NO"
   let vDPSBreak = $FALSE
else
   let vDPSBreak = $TRUE
endif

if $dirpath() not like "%$compdevcen%"
   access $compdevcen
endif
clear
output $center($tochar("Exporting tool to convert ZIM7 databases to ZIM8.",80))
output $center($tochar("==================================================",80))
output

output '    Computing the maximum file number needed...';
let $page = 100
compute objectlist eval(let $page = $max(filenumber))
let $page = $page + 10
out $page

output '    Creating temporary objects'
CreateObjs()

output '    Setting PERUSER atribute to Entitysets and Relationships according AREAS.ZIM'

%%%       NOW EXPORT ALL OBJECTS
output '    Exporting all Objects to the Shadow DD'
%
%%%     FIRST SET ALL OBJECTS IN SHOBJECTLIST TO BE EXPORTED (PROCESSED)
change all Objlist   \
   where objectname not like "$$$%$$$"  and objectname <> "ZIM"\
     let ToProcess = "Y" -> ZOMToProcessSet
%
%%      NOW CLEAR THE SHADOW DICTIONARY AND THE REPORT FILE
pZOMClearShDD()
plClearZ50()
set out $$$CNV1Report$$$
set out terminal
%%      AND THEN EXPORT DD OBJECTS TO SHADOW DD
out " EXPORTING DD OBJECTS TO SHADOW DD"
pZOMExportSet()
out " DD OBJECTS EXPORTED TO SHADOW DD"
set info off

%%      CHECK RELATIONSHIPS AND ROLES POSSIBLE AMBIGUITY
plCheckAmbiguity()

%           SET A PROPERTY (TMPCOUNTER1) IN OBJECTLIST FOR ALL PERUSER ENTS AND DATARELS
%         PERUSER objects are marked with 9999 in TMP1COUNTER of ObjectList
   find $$$Areas$$$ where $pos(text,")") > 0 -> s$$$Areas$$$
   set output ZOMOutputDoc
   select objectname, filenumber from shObjectList,s$$$Areas$$$  \
      where  $tonum($left(text,$pos(Text," ")),0) = shObjectList.filenumber \
      -> s$$$ObjAreas$$$
   set out terminal
   change all s$$$ObjAreas$$$ let shObjectList.Tmpcounter1 = 9999
%

%%% Directories %%%%
output '    Adapting Directories.'

find shDirs wh DirName <> 'ZIM' sorted by foreign dirid -> sDirsNoZim
Change all sDirsNoZim let DirId = $count(9) + {1 where foreign ="NO",10}
%%%
%%%      NOW LETS DO OTHER DD ADAPTATIONS
%%%
output '    Adapting Formfields.'
%%%     Modifications proposed by XADM are numbered
% -------------------------------------------------
% 2. Altera campos ListBox para ComboBox
% --------------------------------------
change all shffs where FT="L" and wdgSubClass <> 4 let FT="C"
%%%   DM = {"I" wh $position(DM,"I") >0, "N"} wdgEditOptions=6 vr="N"     %%% Adaptado pela XADM VERIFICAR ****

% 3. Altera campos ComboBox Always Dropped para ListBox                    %%%   OK
% -----------------------------------------------------
%change all shffs where FT="C" and wdgEditOptions=4 and wdgFont = "Courier New" and wdgFontSize = "9" let FT="L" ->schForm2 ** errado        %%%   Livio 08-Nov-17
change all shffs where FT="C" and WdgStyle = 1  let FT="L" ->schForm2               %%%   Livio 08-Nov-17

% 4. Altera as bordas dos frames                                            %%%% PRECISA VERIFICAR SE AINDA � NECESSARIO
% ------------------------------
change all shffs where FT="F" and WdgBorderStyle = "5" let WdgBorderStyle = "2" FC = "39" ->schForm3

% 5. Altera a cor da escrita do frame de branco para cor da janela            %%%% PRECISA VERIFICAR SE AINDA � NECESSARIO
% ----------------------------------------------------------------
%change all shffs where ft="F" and fc <> "39" let fc = "39" ->schForm4

%% Adjust ENTRYFIELD with wrong fc/bc for the right one most used for EntryFields
change all shffs where bc=30 and fc=36 let bc=17  -> sffs2

%% Remove the atribute "H" (HIGHLIGHT) from DM
change all shffs where DM like "%R%" \
   let shffs.DM = $translate($translate(DM,"RH","N"),"R","N") -> sffs3

%% Remove the atribute "R" (REVERSE) from DM
%%change all shffs \
%%   let shffs.DM = $translate($translate($translate($translate($translate($translate($translate(\
%%                $translate(DM,"HU","H"),"IH","I"),"NH","N"),"NHU","NU"),"NHX","NX"),"NIH","NI"),"NIX","NI"),"RH","R") -> sffs4
%%
%% Adjust ENTRYFIELD with wrong fc/bc for the right one most used for EntryFields
change all shffs let wdgfcred   = {1,wdgfcred}   wdgbcred   = {1,wdgbcred}   \
                 wdgfcgreen = {1,wdgfcgreen} wdgbcgreen = {1,wdgbcgreen} \
                 wdgfcblue  = {1,wdgfcblue}  wdgbcblue  = {1,wdgbcblue}  -> sffs5

%% Adjust ENTRYFIELD with wrong fc/bc for the right one most used for EntryFields
change all shffs let wdgfcred   = {1 wh wdgfcred=0}   wdgbcred   = {1 wh wdgbcred=0}   \
                 wdgfcgreen = {1 wh wdgfcgreen=0} wdgbcgreen = {1 wh wdgbcgreen=0} \
                 wdgfcblue  = {1 wh wdgfcblue=0}  wdgbcblue  = {1 wh wdgbcblue=0}  -> sffs6

%% Adjust Listbox and Combo Box with new WDGHGHT value
change all shffs where (FT in ("L","C") and WdgItems > "") \
      let WdgHgHt = {315 wh wdgStyle = 1,$length($trim(wdgItems))}

%% CHANGE TABSTOP and TABORDER - FIX WRONG BEHAVIOR ON PREVIOUS VERSION OF ZIM 8.X
change all shffs where  WdgTabStop = 0 and FT not in ("G","P") let WdgTabStop = 1 ffsn = fnum

%% ADJUST LISTBOX AND COMBO BOX WITH NEW WDGHGHT VALUE
change all shffs where FT in ("C","L") and WdgStyle=2 let WdgHgHt=315

%% Change entry fields wiht font size <6 to Fontsize 8  (LT)
change all shffs where FT = "V" and wdgFontSize <6  let wdgFontSize =8

%%   Adapt blank fieldnames - "FNameAdoptetdXXXX"  where "XXXX" a sequence from 1 ro the number of blank Fieldnames
change all shffs where FieldName = '' let fieldname= $squeeze("","FNameAdopted",$count(9))

%%   Adapt FFS WdgOnImage containing *.BMP to blank
change all shffs where WdgOnImage like '%*.bmp' let WdgOnImage = ''

%%   Formfields wdgROW or wdgCOL with Negative values - set to zero
change all shffs where wdgRow < 0 let wdgROW = 1
change all shffs where wdgCol < 0 let wdgCOL = 1


% 8. Altera campos int, longint, vastint e numeric com length = 0
% ----------------------------------------------------------------------------------------
change all shfields wh shfields.Type = "VastInt" AND length = 0 let Length = 15
change all shfields wh shfields.Type = "LongInt" AND length = 0 let Length = 9
change all shfields wh shfields.Type = "int"     AND length = 0 let Length = 5

%%%. Inclui entre aspas as strings default dos Fields que estavam sem aspas
%%%    Select the strings that contains Alphanumeric then remove the only numeric
find all shfields where type in ("CHAR","ALPHA","VARCHAR","VARALPHA")  and \
   $isalpha($trim(default)) = $TRUE -> shFLDAlpha
find all shfields where type in ("CHAR","ALPHA","VARCHAR","VARALPHA")  and \
   $isalphanumeric($trim(default)) = $TRUE -> shFLDAlphaNum
find shFLDAlphaNum MINUS shFLDAlpha -> shAlpha1
change all shAlpha1 let Default = $concat('"',Default,'"')
%%%
%%%         MODIFICATIONS FROM THE OLD ZIM8CNV2
%%%
out '    Cleaning up Zim Data Dictionary'

%% CHANGE ORACLESAM (discontinued) to JDBCSAM
change all shents let enttype = {"JDBCSAM" where enttype in ("ORACLE", "SQLSERV","POSTGRES"), enttype}
let vZOMInt1 = $setcount
change all shrels let reltype = {"JDBCSAM" where reltype in ("ORACLE", "SQLSERV","POSTGRES"), reltype}
if vZOMInt1 > 0 or $Membercount > 0
   if $ZIMOS IN (11, 7)
      system "!echo JDBCSAM > EXISTEJDBCSAM" invisible remote
   else
      system "echo JDBCSAM > EXISTEJDBCSAM"  invisible remote
   endif
endif
%%%         If There are enttypes or reltype <> JDBCSAM, ZIM, Blank, Report

set nullvalue "???NULL??NULL???"
set output trim on
find shents where enttype not in ("JDBCSAM","ZIM","") -> sshent1
if $setcount > 0
   set output $$$CNV1Report$$$ append
   out "========================================================="
   out "\0a\0a UNKNOWN ENTTYPE(s) DETECTED DURING THE MIGRATION"
   select EntType, $tochar(count(9),6) Head "Count" from  sshent1 group by enttype
   set out terminal
endif
find shrels where reltype not in ("JDBCSAM","ZIM"," ") -> sshrel1
if $setcount > 0
   set output $$$CNV1Report$$$ append
   out "========================================================="
   out "\0a\0a UNKNOWN RELTYPE(s) DETECTED DURING THE MIGRATION"
   select relType, $tochar(count(9),6) Head "Count" from sshrel1 group by reltype
   set out terminal
endif

%%%     Test if there is at least one TABBED FORM and report
find shffs where fieldname in ("TabBase","Tab1","TabLabel1") -> STABBED
if $setcount > =3
   find shffs where fieldname = "TabLabel"? -> STABBED
   set output $$$CNV1Report$$$ append
   out "===================================================================="
   out "\0a\0a  THERE IS AT LEAST ONE TABBED FORM = CONVERT AFTER MIGRATTION"
   list all sTabbed format formname fieldname default
   set out terminal
endif

%% ADJUST AVGSIZE WHERE ENTTYPE/RELTYPE = JDBCSAM
output '    Adjust Avgsize (Jdbcsam) and Field Sizes.'
change all shents let avgsize = { 0 where enttype = "JDBCSAM", avgsize }
change all shrels let avgsize = { 0 where reltype = "JDBCSAM", avgsize }

%% ADJUSTING THE SIZE AND DECIMALS OF THE FIELDS
change all shEnts shEnt2shFld shFields where Type = "int" and (Length < 1 or Length > 5) \
   let Length = {5 where EntType <> "JDBCSAM", 5 where Decimals = 0, 4}
change all shEnts shEnt2shFld shFields where Type = "longint" and (Length < 1 or Length > 10) \
   let Length = {10 where EntType <> "JDBCSAM", 10 where Decimals = 0, 9}
change all shEnts shEnt2shFld shFields where Type = "vastint" and (Length < 1 or Length > 15) \
   let Length = {15 where EntType <> "JDBCSAM", 15 where Decimals = 0, 14}
change all shRels shRel2shFld shFields where Type = "int" and (Length < 1 or Length > 5) \
   let Length = {5 where RelType <> "JDBCSAM", 5 where Decimals = 0, 4}
change all shRels shRel2shFld shFields  where Type = "longint" and (Length < 1 or Length > 10) \
   let Length = {10 where RelType <> "JDBCSAM", 10 where Decimals = 0, 9}
change all shRels shRel2shFld shFields  where Type = "vastint" and (Length < 1 or Length > 15) \
   let Length = {15 where RelType <> "JDBCSAM", 15 where Decimals = 0, 14}

%% RELATIONSHIPS WITH NO FIELDS ARE OF THE ZIM TYPE
change all shRels (unrelated) ShRel2ShFld shFields \
   let RelType = "ZIM"
change all shRels where {RelType, ""} = "" let RelType = "ZIM"
change all shEnts where {EntType, ""} = "" let EntType = "ZIM"
%%%
%%%         MODIFICATIONS INTRODUCED BY ALEX
%%%
%% Changing WINDOWS association to the only one existing in WINS
find all shwins sorted by winname dirname -> swins
while
   let <winname> = winname \
       <windirname> = dirname
   change all shpaintergroups where winname="#<winname>" let windirname="#<windirname>"
   set currentset swins
   if $currentmember("swins") = $lastmember("swins")
      break
   endif
   next swins
endwhile


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  All Done!  No we can generate the DCZ% files.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

out "    Generating the Z50 files."
plExportSD()

set reset
set information off
set membercount off

%  NOW LET'S ADAPT DCZ50ENTS AND DCZ50RELS ADDING THE PERUSER FIELD
output '    Adapt DCz50Ents and DCz50Rels adding the Peruser Field'
set save
set headings off
set output trim on
set output DPSDocViewData
select $squeeze("",$$$z50ents$$$.text,{":^YES^" where shObjectList.Tmpcounter1 = 9999,":^NO^"}) from shObjectlist,$$$z50ents$$$  \
   where shObjectlist.objecttype='ENT'? \
     and shObjectlist.objectname=$substring($$$z50ents$$$.text,2,$pos($substring($$$z50ents$$$.text,2,25),'^')-1)
set output dcz50ents
li all DPSDocViewData

set output DPSDocViewData
select $squeeze("",$$$z50rels$$$.text,{":^YES^" where shObjectList.Tmpcounter1 = 9999,":^NO^"}) from shObjectlist,$$$z50rels$$$ \
   where shObjectlist.objecttype='REL'? \
     and shObjectlist.objectname=$substring(text,2,$pos($substring(text,2,25),'^')-1)
set output dcz50rels
list all DPSDocViewData

change all shdocs ShDoc2ShObjList shobjlist where shdocs.filename like ")%" let shobjlist.Tmpcounter2 = 9999
set output DPSDocViewData
select $squeeze("",$$$z50docs$$$.text,{":^YES^" where shObjectList.Tmpcounter2 = 9999,":^NO^"}) \
from shObjectlist,$$$z50docs$$$ \
   where shObjectlist.objecttype='DOC'? \
     AND  shObjectlist.objectname=$substring(text,2,$pos($substring(text,2,25),'^')-1)  \
     and shObjectlist.objectkey LIKE (LET VOBJECTKEY =$translate(($delete($right($trim(text),27),1,$pos($right($trim(text),27),":"))),"^",''))
set output dcz50docs
list all DPSDocViewData
set restore
set out terminal

%=======================================================================================================================
%%% Meta Objects.
plExpMetaObjs()

%=======================================================================================================================

set reset
zomdestroy $$$*$$$
let vZOMerrorlog = $TRUE
let vZOMDisplayActive = $TRUE
let vZOMDisplay = 'S'
out "     All Done."
if vDPSBreak = $TRUE
   bye
endif

