create or replace PACKAGE "ZIMFUNC" AS

   FUNCTION ISALPHABETIC(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISALPHANUMERIC(value IN NUMBER)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISALPHANUMERIC(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISDIGIT(value VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISNUMBER(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISLOWER(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISUPPER(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION ISDATE(value IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION DATEDAY(value IN DATE)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION DATEMONTH(value IN DATE)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION DATEYEAR(value IN DATE)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION MAXOF(value IN VARCHAR, value2 IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION MINOF(value IN VARCHAR, value2 IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION RIGHTJUSTIFY(value IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION SUBSTRING(value IN VARCHAR, position IN VARCHAR, length IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION TOCHAR(value IN NUMBER, rlen NUMBER)
      RETURN VARCHAR2 DETERMINISTIC;
   
   FUNCTION TOCHAR(value IN DATE, rlen NUMBER)
      RETURN VARCHAR2 DETERMINISTIC;
   
   FUNCTION TOCHAR(value IN VARCHAR2, rlen NUMBER)
      RETURN VARCHAR2 DETERMINISTIC;

   FUNCTION TONUM(value IN VARCHAR, scale IN VARCHAR)
      RETURN NUMBER DETERMINISTIC;

   FUNCTION MASK(value IN VARCHAR, mask IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION MASKDATE(value IN VARCHAR, mask IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION MASKSTR(value IN VARCHAR, mask IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION MASKNUM(value IN NUMBER, mask IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION MASKNUM_GETPRE(value IN VARCHAR)
      RETURN VARCHAR DETERMINISTIC;

   FUNCTION ZRTRIM(value IN NUMBER)
      RETURN VARCHAR2 DETERMINISTIC;
   
   FUNCTION ZRTRIM(value IN DATE)
      RETURN VARCHAR2 DETERMINISTIC;
   
   FUNCTION ZRTRIM(value IN VARCHAR2)
      RETURN VARCHAR2 DETERMINISTIC;

END ZIMFUNC;

create or replace PACKAGE BODY "ZIMFUNC" AS

/******************************************************************************************************/
FUNCTION ISALPHABETIC(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
   RC NUMBER;
BEGIN
   IF ((length(REGEXP_REPLACE(value,'[a-zA-Z]', '')) IS NULL) AND (value IS NOT NULL)) 
   THEN
      RETURN 1;
   ELSE
      RETURN 0;
   END IF;
END;

/******************************************************************************************************/
FUNCTION ISALPHANUMERIC(value IN NUMBER) 
   RETURN NUMBER DETERMINISTIC AS
BEGIN
   RETURN 0;
END;

FUNCTION ISALPHANUMERIC(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
BEGIN
   IF ((LENGTH(REGEXP_REPLACE(value,'[a-zA-Z]|[0-9]', '')) > 0) OR (value IS NULL)) 
   THEN
      RETURN 0;
   ELSE
      RETURN 1;
   END IF;
EXCEPTION
WHEN OTHERS THEN
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION ISDIGIT(VALUE IN VARCHAR) RETURN NUMBER DETERMINISTIC AS
BEGIN
   IF ((LENGTH(REGEXP_REPLACE(value,'[0-9]', '')) > 0) OR (value IS NULL))
   THEN
      RETURN 0;
   ELSE
      RETURN 1;
   END IF;
EXCEPTION
WHEN OTHERS THEN
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION ISNUMBER(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
   RC NUMBER;
BEGIN
   IF (value IS NULL) 
   THEN
      RETURN 0;
   END IF;
   RC := TO_NUMBER(value);
   RETURN 1;
EXCEPTION
WHEN OTHERS THEN
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION ISLOWER(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
BEGIN
   IF ((length(REGEXP_REPLACE(value, '[a-z]', '')) > 0) OR (value IS NULL)) 
   THEN
      RETURN 0;
   ELSE
      RETURN 1;
   END IF;
END;

/******************************************************************************************************/
FUNCTION ISUPPER(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
BEGIN
   IF ((length(REGEXP_REPLACE(value, '[A-Z]', '')) > 0)OR (value IS NULL)) 
   THEN
      RETURN 0;
   ELSE
      RETURN 1;
   END IF;
END;

/******************************************************************************************************/
FUNCTION ISDATE(value IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
   iDate DATE;
BEGIN
   iDate := TO_DATE(value);
   RETURN 1;
EXCEPTION
WHEN OTHERS THEN
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION DATEDAY(value IN DATE) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN LPAD(TO_CHAR(value,'FMdd'),2);
END;

FUNCTION DATEMONTH(value IN DATE) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN LPAD(TO_CHAR(value,'FMmm'),2);
END;

FUNCTION DATEYEAR(value IN DATE) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN LPAD(TO_CHAR(value,'FMyy'),2);
END;

/******************************************************************************************************/
FUNCTION MAXOF(value IN VARCHAR, value2 IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
   iLevel NUMBER(15);
BEGIN
   IF (ISNUMBER(value) = 1 AND ISNUMBER(value2) = 1)
   THEN
      IF (TO_NUMBER(value) > TO_NUMBER(value2)) 
      THEN
         RETURN value;
      ELSE
         RETURN value2;
      END IF;
   ELSIF (ISNUMBER(value) = 0 AND ISNUMBER(value2) = 0)
   THEN
      iLevel := 0;
   ELSIF (ISNUMBER(value) = 0) 
   THEN
      iLevel := value2;
   ELSE
      iLevel := value;
   END IF;
   RETURN iLevel;
EXCEPTION
WHEN OTHERS THEN
   dbms_output.put_line('***Error*** The string is not a valid number. The value has been set to zero.');
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION MINOF(value IN VARCHAR, value2 IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
   iLevel NUMBER(15);
BEGIN
   IF (ISNUMBER(value) = 1 AND ISNUMBER(value2) = 1)
   THEN
      IF TO_NUMBER(VALUE) < TO_NUMBER(VALUE2) 
      THEN
         RETURN value;
      ELSE
         RETURN value2;
      END IF;
   ELSIF (ISNUMBER(value) = 0 AND ISNUMBER(value2) = 0)
   THEN
      iLevel := 0;
   ELSIF (ISNUMBER(value) = 0) THEN
      iLevel := value2;
   ELSE
      iLevel := value;
   END IF;
   RETURN iLevel;
EXCEPTION
WHEN OTHERS THEN
   dbms_output.put_line('***Error*** The string is not a valid number. The value has been set to zero.');
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION RIGHTJUSTIFY(value IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN LPAD(RTRIM(value),LENGTH(value),' ');
END;

/******************************************************************************************************/
FUNCTION SUBSTRING(value IN VARCHAR, position IN VARCHAR, length IN VARCHAR ) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN SUBSTR(to_char(cast(value AS TIMESTAMP), 'YYYYMMDD'), position, length);
EXCEPTION
WHEN OTHERS THEN
   RETURN SUBSTR(value, position, length);
END;

/******************************************************************************************************/
FUNCTION TOCHAR(value IN VARCHAR2, rlen NUMBER) 
   RETURN VARCHAR2 AS
BEGIN
   IF (RPAD(RTRIM(value),rlen,' ') IS NULL)
   THEN
      RETURN RPAD(value,rlen,' '); 
   ELSE
      RETURN RPAD(RTRIM(value),rlen,' ');
   END IF;   
END;

FUNCTION TOCHAR(value IN DATE, rlen NUMBER) 
   RETURN VARCHAR2 AS
BEGIN
   RETURN LPAD(RTRIM(value),rlen,' ');
END;

FUNCTION TOCHAR(value IN NUMBER, rlen NUMBER) 
   RETURN VARCHAR2 AS
BEGIN
   RETURN LPAD(RTRIM(value),rlen,' ');
END;

/******************************************************************************************************/
FUNCTION TONUM(value IN VARCHAR, scale IN VARCHAR) 
   RETURN NUMBER DETERMINISTIC AS
BEGIN
   IF (value = ' ')
   THEN
      RETURN 0;
   ELSE
      RETURN round(value, scale);
   END IF;
EXCEPTION
WHEN OTHERS THEN
   dbms_output.put_line('***Error*** the string '||value||' is not a valid number. The value has been set to zero.');
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION MASKDATE(value IN VARCHAR, mask IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
BEGIN
   RETURN TO_CHAR(TO_DATE(value),mask) ;
EXCEPTION
WHEN OTHERS THEN
   dbms_output.put_line('***Error*** Could not mask ' || value || mask || '. value set to mask');
   RETURN '>' || mask;
END;

/******************************************************************************************************/
FUNCTION MASKSTR(value IN VARCHAR, mask IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
   RC VARCHAR(100);
   CH VARCHAR(1);
   j NUMBER;
BEGIN
   j := 1;
   FOR i IN 1..LENGTH(mask) LOOP
      CH := SUBSTR(mask, i, 1);
      IF (CH  = '?') THEN
         RC := RC||SUBSTR(value, j, 1);
         j := j +1;
      ELSE
         RC := RC||CH;
      END IF;
   END LOOP;
   RETURN RC;
END;

/******************************************************************************************************/
FUNCTION MASKNUM_GETPRE(value IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
   j NUMBER;
BEGIN
   IF (INSTR(value, '.',1,2) <> 0) 
   THEN
      dbms_output.put_line('***Error*** A character or numaric field is too small to hold the number. The result has been set to "0"s.');
      RETURN 0;
   END IF;
   j := INSTR(value, '.');
   IF (j <> 0) 
   THEN
      RETURN SUBSTR(value, j, LENGTH(value));
   ELSE
      RETURN ' ';--Had to add this because ORACLE seems to have issues with NULL's or ''.
   END IF;
END;

/******************************************************************************************************/
FUNCTION MASKNUM_REVERSE(value IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
   j    NUMBER;
   Temp VARCHAR(32767);
BEGIN
   FOR i IN 1..LENGTH(value) LOOP
      Temp := substr(value, i, 1)||Temp;
   END LOOP;
   RETURN Temp;
END;

/******************************************************************************************************/
FUNCTION MASKNUM(value IN NUMBER, mask IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
   VSize    NUMBER;
   MSize    NUMBER;
   IsPos    VARCHAR2(1);
   HASSIGN  VARCHAR2(1);
   NewMask  VARCHAR2(256);
BEGIN
   IF ((INSTR(Mask, '.',1,2) <> 0) OR (INSTR(value, '.',1,2) <> 0)) 
   THEN
      dbms_output.put_line('***Error*** A character or numaric field is too small to hold the number. The result has been set to "0"s.');
      RETURN 0;
   END IF;
   IF TRANSLATE('ZZZ999.,$-+','!9Zz$.,*+-','@') IS NOT NULL
   THEN  
      dbms_output.put_line('***Error*** Invalid Characters in the mask patern: ' || Mask);
      RETURN '***Error*** Invalid Characters in the mask patern: ' || Mask;
   END IF;      
   HasSign  := '0';
   VSize    := LENGTH(value);
   MSIZE    := length(MASK);
   NewMask  := UPPER(Mask);
   IF (value < 0)
   THEN 
      IsPos := '0';  -- value is negative;
   ELSE 
      IsPos := '1';  -- value is positive
   END IF;
   IF (SUBSTR(MASK,1,1) IN ('+','-') AND IsPos = 0)
   THEN 
      NewMask := 'S' || SUBSTR(NewMask,2); 
      HasSign := '1';
   END IF;
   NEWMASK := TRANSLATE(NEWMASK,'Z9-,.','909GD');
   -- return NEWMASK;
   IF (HASSIGN = '0')
   THEN 
      RETURN SUBSTR(TO_CHAR(value,NEWMASK),2);
   ELSE 
      RETURN TO_CHAR(value,NEWMASK);
   END IF;
END ;

/******************************************************************************************************/
FUNCTION MASK(value IN VARCHAR, mask IN VARCHAR) 
   RETURN VARCHAR DETERMINISTIC AS
   RC VARCHAR(100);
   CH VARCHAR(1);
   j NUMBER;
BEGIN
   IF (LENGTH(REGEXP_SUBSTR(mask,'MM|DD|YY|YYYY')) > 0) 
   THEN
      RC := TO_CHAR(TO_DATE(value),mask);
   ELSE
      IF (INSTR(mask, '?',1,1) <> 0) 
      THEN
         RC := MASKSTR(value , mask);
      ELSE
         RC := MASKNUM(value , mask);
      END IF;
   END IF;
   RETURN RC;
EXCEPTION 
WHEN OTHERS THEN
   dbms_output.put_line('***Error*** A character or numaric field is too small to hold the number. The result has been set to "0"s.');
   RETURN 0;
END;

/******************************************************************************************************/
FUNCTION ZRTRIM(value IN VARCHAR2) 
   RETURN VARCHAR2 AS
BEGIN
   IF (VALUE IS NULL) 
   THEN
      RETURN VALUE;
   END IF;   
   IF (LENGTH(RTRIM(VALUE)) IS NULL)
   THEN
      RETURN ' ';
   END IF;   
   IF (LENGTH(RTRIM(value)) > 0) OR (value is NOT NULL)
   THEN
      RETURN RTRIM(VALUE);
   END IF;
END;

FUNCTION ZRTRIM(value IN DATE) 
   RETURN VARCHAR2 AS
BEGIN
   IF (VALUE IS NULL) 
   THEN
      RETURN VALUE;
   END IF;   
   IF (LENGTH(RTRIM(VALUE)) IS NULL)
   THEN
      RETURN ' ';
   END IF;   
   IF (LENGTH(RTRIM(value)) > 0) OR (value is NOT NULL)
   THEN
      RETURN RTRIM(VALUE);
   END IF;
END;

FUNCTION ZRTRIM(value IN NUMBER) 
   RETURN VARCHAR2 AS
BEGIN
   IF (VALUE IS NULL) 
   THEN
      RETURN VALUE;
   END IF;   
   IF (LENGTH(RTRIM(VALUE)) IS NULL)
   THEN
      RETURN ' ';
   END IF;   
   IF (LENGTH(RTRIM(value)) > 0) OR (value is NOT NULL)
   THEN
      RETURN RTRIM(VALUE);
   END IF;
END;
/******************************************************************************************************/

END;